#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# xwot.py - Python tools for the extended Web of Things
# Copyright (C) 2015  Alexander Rüedlinger
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

__author__ = 'Alexander Rüedlinger'

import struct
import smbus


class Adapter(object):

    def __init__(self, bus=1, i2c_addr=0x04):
        self._bus = smbus.SMBus(bus)
        self._i2c_addr = i2c_addr

    @property
    def bus(self):
        return self._bus

    @property
    def i2c_addr(self):
        return self._i2c_addr

    def _to_int(self, data):
        """
        Assumes that data is in big endian format: [msb, ..., lsb]
        """
        b = ''.join(chr(i) for i in data)
        r = struct.unpack('>i', b)
        return r[0]

    def _to_uint(self, data):
        """
        Assumes that data is in big endian format: [msb, ..., lsb]
        """
        b = ''.join(chr(i) for i in data)
        r = struct.unpack('>I', b)
        return r[0]

    def _to_short(self, data):
        """
        Assumes that data is in big endian format: [msb, ..., lsb]
        """
        b = ''.join(chr(i) for i in data)
        r = struct.unpack('>h', b)
        return r[0]

    def _to_ushort(self, data):
        """
        Assumes that data is in big endian format: [msb, ..., lsb]
        """
        b = ''.join(chr(i) for i in data)
        r = struct.unpack('>H', b)
        return r[0]

    def _to_float(self, data):
        """
        Assumes that data is in big endian format: [msb, ..., lsb]
        """
        b = ''.join(chr(i) for i in data)
        r = struct.unpack('>f', b)
        return r[0]

    def _to_double(self, data):
        """
        Assumes that data is in big endian format: [msb, ..., lsb]
        """
        b = ''.join(chr(i) for i in data)
        r = struct.unpack('>d', b)
        return r[0]

    def write_byte(self, reg):
        try:
            self._bus.write_byte(self._i2c_addr, reg)
            return True
        except IOError:
            return False

    def write_word(self, reg, val):
        try:
            self._bus.write_word_data(self._i2c_addr, reg, val)
            return True
        except IOError:
            return False

    def read_byte(self, reg):
        try:
            return self._bus.read_byte_data(self._i2c_addr, reg)
        except IOError:
            return None

    def read_bytes(self, reg, byte_count):
        try:
            return self._bus.read_i2c_block_data(self._i2c_addr, reg, byte_count)
        except IOError:
            return None

    def read_int32(self, reg):
        data = self.read_bytes(reg, 4)
        if data is not None:
            return self._to_int(data)
        else:
            return None

    def read_uint32(self, reg):
        data = self.read_bytes(reg, 4)
        if data is not None:
            return self._to_uint(data)
        else:
            return None

    def read_int16(self, reg):
        data = self.read_bytes(reg, 2)
        if data is not None:
            return self._to_short(data)
        else:
            return None

    def read_uint16(self, reg):
        data = self.read_bytes(reg, 2)
        if data is not None:
            return self._to_ushort(data)
        else:
            return None

    def read_float(self, reg):
        data = self.read_bytes(reg, 4)
        if data is not None:
            return self._to_float(data)
        else:
            return None

    def read_double(self, reg):
        data = self.read_bytes(reg, 8)
        if data is not None:
            return self._to_double(data)
        else:
            return None


import os
from flask import Flask
from flask import Response
import models
from xwot.util.vocab import Hydra
from xwot.util.vocab import Owl
from xwot.util.flask import hydra_link
from xwot.util import local_ip
from xwot.util import create_description
from xwot.util import dir_path

from xwot.util.flask import mount_apidoc
from xwot.util.flask import make_response
from xwot.util.vocab import Xwot

from models import annotate

# base config
ip = local_ip()
port = 3000
http_addr = "http://%s:%s/" % (ip, port)
apidoc_url = http_addr + 'vocab'
hydra_link = hydra_link(apidoc_url)


jsonld_description = create_description(xwot_file=os.path.join(dir_path(__file__), "device.xwot"), base=http_addr)


annotate.documentation(entrypoint=http_addr, title='My Api Doc', description='Awesome xWoT Api documentation.',
                       apidoc_url=apidoc_url)

app = Flask(__name__, static_path='')


entrypoint = models.EntryPoint()

"""
Entrypoint controller
"""

"""
    GET EntryPoint
"""
@annotate.route('entry_point', method='GET', description='The APIs main entry point.', returns=Xwot.Resource())
@app.route('/', methods=['GET'])
@hydra_link
def entrypoint_get():
    return make_response(entrypoint)


"""
User controller
"""

"""
    GET User collection
"""
@annotate.route('user_collection_retrieve', method='GET', description='Retrieves all User entities', returns=Hydra.Collection())
@app.route('/users', methods=['GET'])
@hydra_link
def get_user_collection():
    users = models.user_collection
    return make_response(users)




"""
    POST User
"""
@annotate.route('user_create', method='POST', description='Creates a new User entity', returns=Xwot.Resource(),
                expects=Xwot.Resource(), status_codes=[{"code": 201, "description": "If the User entity was created successfully."}])
@app.route('/users', methods=['POST'])
@hydra_link
def post_user(user_id):
    user = models.user_collection.user(user_id)
    if user:
        return make_response(user)
    else:
        return Response(status=404)
"""
    PUT User
"""
@annotate.route('user_replace', method='PUT', description='Replaces a new User entity', returns=Xwot.Resource(),
                expects=Xwot.Resource(), status_codes=[{"code": 201, "description": "If the User entity was created successfully."}])
@app.route('/users', methods=['PUT'])
@hydra_link
def put_user(user_id):
    user = models.user_collection.user(user_id)
    if user:
        return make_response(user)
    else:
        return Response(status=404)


"""
    GET User
"""
@annotate.route('user_retrieve', method='GET', description='Retrieves a User entity', returns=Xwot.Resource(),
                 status_codes=[{"code": 404, "description": "If the User entity wasn't found."}])
@app.route('/users/<int:user_id>', methods=['GET'])
@hydra_link
def get_user(user_id):
    user = models.user_collection.user(user_id)
    if user:
        return make_response(user)
    else:
        return Response(status=404)


"""
    DELETE user
"""
@annotate.route('user_delete', method='DELETE', description='Deletes a User entity', returns=Owl.Nothing(),
                 status_codes=[{"code": 404, "description": "If the User entity wasn't found."}])
@app.route('/users/<int:user_id>', methods=['DELETE'])
@hydra_link
def delete_user(user_id):
    user = models.user_collection.user(user_id)
    if user:
        return ''
    else:
        return Response(status=404)


from xwot.util.apidocbuilder import HydraBuilder
builder = HydraBuilder(annotate)
mount_apidoc(app, builder)
app.debug = True
app.run(host='0.0.0.0', port=port)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# xwot.py - Python tools for the extended Web of Things
# Copyright (C) 2015  Alexander Rüedlinger
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

__author__ = 'Alexander Rüedlinger'
import copy


class Visitable:

    def accept(self, node):
        raise NotImplementedError


class Visitor(object):
    def visit_entity(self, node):
        raise NotImplementedError

    def visit_resource(self, node):
        raise NotImplementedError

    def visit_device_resource(self, node):
        raise NotImplementedError

    def visit_sensor_resource(self, node):
        raise NotImplementedError

    def visit_service_resource(self, node):
        raise NotImplementedError

    def visit_actuator_resource(self, node):
        raise NotImplementedError

    def visit_tag_resource(self, node):
        raise NotImplementedError

    def visit_context_resource(self, node):
        raise NotImplementedError

    def visit_publisher_resource(self, node):
        raise NotImplementedError

    def visit_publisher_client_resource(self, node):
        raise NotImplementedError

    def visit_device(self, node):
        raise NotImplementedError

    def visit_sensor(self, node):
        raise NotImplementedError

    def visit_actuator(self, node):
        raise NotImplementedError

    def visit_tag(self, node):
        raise NotImplementedError


class BaseVisitor(Visitor):

    def __init__(self):
        self._entity_node = None
        self._nodes = []

    def before(self):
        pass

    def after(self):
        pass

    def before_resource(self, node):
        pass

    def after_resource(self, node):
        pass

    def visit_entity(self, node):
        #node.set_name('RootResource')
        self._entity_node = node
        self._nodes.append(node)
        self.before()
        self.handle_entity(node)
        [child.accept(self) for child in node.children() if child.is_virtual()]
        self.after()

    def visit_resource(self, node):
        self._nodes.append(node)
        self.before_resource(node)
        self.handle_resource(node)
        self.after_resource(node)
        for child in node.children():
            child.accept(self)

    def visit_device_resource(self, node):
        self._nodes.append(node)
        self.before_resource(node)
        self.handle_device_resource(node)
        self.after_resource(node)
        for child in node.children():
            child.accept(self)

    def visit_sensor_resource(self, node):
        self._nodes.append(node)
        self.before_resource(node)
        self.handle_sensor_resource(node)
        self.after_resource(node)
        for child in node.children():
            child.accept(self)

    def visit_service_resource(self, node):
        self._nodes.append(node)
        self.before_resource(node)
        self.handle_service_resource(node)
        self.after_resource(node)

    def visit_actuator_resource(self, node):
        self._nodes.append(node)
        self.before_resource(node)
        self.handle_actuator_resource(node)
        self.after_resource(node)
        for child in node.children():
            child.accept(self)

    def visit_tag_resource(self, node):
        self._nodes.append(node)
        self.before_resource(node)
        self.handle_tag_resource(node)
        self.after_resource(node)
        for child in node.children():
            child.accept(self)

    def visit_context_resource(self, node):
        self._nodes.append(node)
        self.before_resource(node)
        self.handle_context_resource(node)
        self.after_resource(node)
        for child in node.children():
            child.accept(self)

    def visit_publisher_resource(self, node):
        self._nodes.append(node)
        self.before_resource(node)
        self.handle_publisher_resource(node)
        self.after_resource(node)

        node2 = copy.deepcopy(node)
        nodename = 'ClientResource'.join(node2.name().rsplit('Resource', 1))
        if node.name() == nodename:
            nodename += 'Client'
        node2.set_name(nodename)
        node2.set_fullpath(node2.fullpath()+'/<clientid>')
        self._nodes.append(node2)
        self.before_resource(node2)
        self.handle_publisher_client_resource(node2)
        self.after_resource(node2)



    def visit_device(self, node):
        pass

    def visit_sensor(self, node):
        pass

    def visit_actuator(self, node):
        pass

    def visit_tag(self, node):
        pass

    def handle_entity(self, node):
        pass

    def handle_resource(self, node):
        pass

    def handle_device_resource(self, node):
        pass

    def handle_sensor_resource(self, node):
        pass

    def handle_tag_resource(self, node):
        pass

    def handle_context_resource(self, node):
        pass

    def handle_service_resource(self, node):
        pass

    def handle_actuator_resource(self, node):
        pass

    def handle_publisher_resource(self, node):
        pass

    def handle_publisher_client_resource(self, node):
        pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# xwot.py - Python tools for the extended Web of Things
# Copyright (C) 2015  Alexander Rüedlinger
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

__author__ = 'Alexander Rüedlinger'

import os
import socket
from xwot.compiler.frontend.processing import JSONLDDescriptionBuilder
import xmltodict
import json


def local_ip():
    # source: http://stackoverflow.com/questions/166506/finding-local-ip-addresses-using-pythons-stdlib
    return [(s.connect(('8.8.8.8', 80)), s.getsockname()[0], s.close()) for s
            in [socket.socket(socket.AF_INET, socket.SOCK_DGRAM)]][0][1]


def create_description(xwot_file, site='', base=None):
    description_builder = JSONLDDescriptionBuilder()
    out = description_builder.build(xml_file=xwot_file, site=site, base=base)
    return out


def dir_path(file):
    return os.path.dirname(os.path.realpath(file))


def parent_dir_path(file):
    return os.path.dirname(dir_path(file))



CONTENT_TYPES = {
    'application/xml': xmltodict.parse,
    'application/json': json.loads,
    'application/ld+json': json.loads
}


def deserialize(data, content_type):
    _list = content_type.split(';')  # parse content type headers e.g.: application/json;charset=UTF-8
    try:
        _content_type = _list[0]
        if _content_type in CONTENT_TYPES and data:
            deserializer = CONTENT_TYPES[_content_type]
            res = deserializer(data)
            return res
        else:
            return {}
    except:
        return {}


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# xwot.py - Python tools for the extended Web of Things
# Copyright (C) 2015  Alexander Rüedlinger
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

__author__ = 'Alexander Rüedlinger'

import sys
import os
import os.path

from backend.KleinBackendBuilder import KleinBackendBuilder
from backend.FlaskBackendBuilder import FlaskBackendBuilder
from backend.SinatraBackendBuilder import SinatraBackendBuilder
from backend.ExpressBackendBuilder import ExpressBackendBuilder
from frontend.parser import Parser
from frontend.processing import JSONLDDescriptionPrinter
from frontend.processing import TreeCleaner


class Compiler(object):

    BACKENDS = {
        'flask': FlaskBackendBuilder,
        'klein': KleinBackendBuilder,
        'express': ExpressBackendBuilder,
        'sinatra': SinatraBackendBuilder
    }

    def __init__(self, input_file, output_dir, platform):
        self._xwot_file = input_file
        self._output_dir = output_dir
        self._platform = platform

    def _parse_input_file(self, file_path):
        # parse xml xwot file
        parser = Parser()
        root_node = parser.parse(file_path)
        return root_node

    def _check_input_file(self, file_path):
        # check if input file exists
        if not os.path.exists(file_path):
            print("Input file does not exist: %s " % file_path)
            sys.exit(1)

    def _clean_tree(self, root_node):
        cleaner = TreeCleaner()
        root_node.accept(cleaner)

    def _build_backend(self, root_node):
         # select backend builder
        backend_builder = self.BACKENDS[self._platform]
        builder = backend_builder()

        # build backend
        root_node.accept(builder)

        for file_name, code in builder.output().items():
            file_name = os.path.join(self._output_dir, file_name)
            path = os.path.dirname(file_name)

            # if path does not exit
            if not os.path.exists(path):
                os.makedirs(os.path.dirname(file_name))

            with open(file_name, 'w+') as f:
                f.write(code)

    def _create_description(self, root_node):
        file_name = os.path.join(self._output_dir, 'description.jsonld')
        description_printer = JSONLDDescriptionPrinter('')
        root_node.accept(description_printer)
        out = description_printer.output()

        with open(file_name, 'w+') as f:
            f.write(out)

    def _create_xml_xwot_file(self, xml_filepath):
        xml_str = open(xml_filepath, "r").read()
        file_name = os.path.join(self._output_dir, 'device.xwot')

        with open(file_name, 'w+') as f:
            f.write(xml_str)

    def compile(self):
        self._check_input_file(self._xwot_file)
        root_node = self._parse_input_file(self._xwot_file)

        self._clean_tree(root_node)
        self._build_backend(root_node)
        self._create_description(root_node)
        self._create_xml_xwot_file(self._xwot_file)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# xwot.py - Python tools for the extended Web of Things
# Copyright (C) 2015  Alexander Rüedlinger
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>


from __future__ import absolute_import

from flask import Response
from datetime import timedelta
from flask import request, current_app
from functools import update_wrapper

__author__ = 'Alexander Rüedlinger'
__all__ = ['cors', 'make_response']


SERIALIZERS = {
    'application/json': lambda obj: obj.to_json(),
    'application/xml': lambda obj: obj.to_xml(),
    'text/plain': lambda obj: obj.to_html(),
    'application/ld+json': lambda obj: obj.to_jsonld()
}


def cors(origin='*', methods=None, headers=None, max_age=2520):
    # source: http://flask.pocoo.org/snippets/56/

    if methods is not None:
        methods = ', '.join(sorted(x.upper() for x in methods))
    else:
        methods = 'GET, OPTIONS'

    if headers is not None:
        headers = ', '.join(headers)
    else:
        headers = 'x-prototype-version,x-requested-with'

    from flask import make_response

    def decorator(f):
        def wrapped_function(*args, **kwargs):
            resp = make_response(f(*args, **kwargs))

            h = resp.headers
            h['Access-Control-Allow-Origin'] = origin
            h['Access-Control-Allow-Methods'] = methods
            h['Access-Control-Max-Age'] = str(max_age)
            h['Access-Control-Allow-Headers'] = headers

            return resp

        f.provide_automatic_options = False
        return update_wrapper(wrapped_function, f)
    return decorator


def make_response(obj, default='application/ld+json', status=200):
    cts = request.accept_mimetypes

    if cts:
        content_type, _ = cts[0]

        if content_type in SERIALIZERS:
            fun_serializer = SERIALIZERS[content_type]

            doc = fun_serializer(obj)
            return Response(response=doc, status=status, content_type=content_type)
        else:
            fun_serializer = SERIALIZERS[default]
            doc = fun_serializer(obj)
            return Response(response=doc, status=status, content_type=default)
    else:
        fun_serializer = SERIALIZERS[default]
        doc = fun_serializer(obj)
        return Response(response=doc, status=status, content_type=default)


import sys
import logging
import time
import json

try:
    from twisted.web import resource
    from twisted.internet import reactor
    from twisted.python import log
    from twisted.web.server import Site
    from twisted.web.static import File
    from autobahn.twisted.resource import WebSocketResource, HTTPChannelHixie76Aware
    from autobahn.twisted.websocket import WebSocketServerFactory, WebSocketServerProtocol, listenWS
except:
    print 'Some dependendencies are not met'
    print 'You need the following packages: twisted, autobahn, websocket'
    print 'install them via pip'
    sys.exit()


class xWoTStreamerProtocol(WebSocketServerProtocol):
    """
    Very basic WebSocket Protocol. All clients are accepted. Furthermore received messages are fowarded to all clients.
    """

    def onOpen(self):
        self.factory.register(self)

    def onMessage(self, payload, isBinary):
        if not isBinary:
            msg = "{} from {}".format(payload.decode('utf8'), self.peer)
            self.factory.broadcast(msg)

    def connectionLost(self, reason):
        WebSocketServerProtocol.connectionLost(self, reason)
        self.factory.unregister(self)


class xWoTBroadcastFactory(WebSocketServerFactory):
    """
    Broadcasts the State at regular intervalls to all connected clients.
    """

    def __init__(self, url, model, debug=False, debugCodePaths=False):
        WebSocketServerFactory.__init__(self, url, debug=debug, debugCodePaths=debugCodePaths)
        self.clients = []
        self.tickcount = 0
        self.lastbroadcast = 0
        self.olddata = ''
        self.model = model
        self.tick()
        self.acquiredata()
        

    def acquiredata(self):
        localdata = self.model.to_json()

        millis = int(round(time.time() * 1000))
        if (localdata != self.olddata) or (millis - self.lastbroadcast) > 10000:
            self.olddata = localdata
            self.lastbroadcast = millis
            try:
                self.broadcast(localdata)
            except TypeError, e:
                logging.error("no value")
                logging.error(e)
            except Exception:
                logging.error("Something bad happend: ")

        reactor.callLater(1, self.acquiredata)

    def tick(self):
        self.tickcount += 1
        self.broadcast("tick %d from server" % self.tickcount)
        reactor.callLater(299, self.tick)

    def register(self, client):
        if not client in self.clients:
            logging.debug('registered client ' + client.peer)
            self.clients.append(client)

    def unregister(self, client):
        if client in self.clients:
            logging.debug("unregistered client " + client.peer)
            self.clients.remove(client)

    def broadcast(self, msg):
        logging.debug("broadcasting prepared message '{}'".format(msg))
        preparedMsg = self.prepareMessage(msg)
        for c in self.clients:
            c.sendPreparedMessage(preparedMsg)
            logging.debug("prepared message sent to {}".format(c.peer))





#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# xwot.py - Python tools for the extended Web of Things
# Copyright (C) 2015  Alexander Rüedlinger
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

__author__ = 'Alexander Rüedlinger'

import json
import dicttoxml
from xml.dom.minidom import parseString


def pretty_json(dic):
    json_doc = json.dumps(dic, indent=4, sort_keys=True, separators=(',', ': '))
    return json_doc


class Entity(object):
    __type__ = 'xwot:Entity'

    __contexts__ = ['http://xwot.lexruee.ch/contexts/xwot']

    __links__ = []
    __expose__ = []

    def __init__(self):
        self._links = []
        self._types = []
        self._contexts = []
        self._resource_path = '/'

    @property
    def resource_path(self):
        return self._resource_path

    @resource_path.setter
    def resource_path(self, value):
        self._resource_path = value

    @property
    def id(self):
        return ''

    @property
    def type(self):
        return self.__type__

    @property
    def types(self):
        return [self.__type__] + self._types

    def add_link(self, name):
        self._links.append(name)

    def add_context(self, context):
        self._contexts += [context]

    def add_type(self, type):
        self._types += [type]

    def _get_exposed_properties(self):
        props = [prop for prop in self.__expose__ if hasattr(self, prop)]
        return props

    def _get_dict(self):
        dic = {}
        for key in self._get_exposed_properties():
            dic[key] = getattr(self, key)
        return dic

    def _enhance_dic(self, dic):
        dic['type'] = self.__type__
        dic['links'] = []

        for link in self._links:
            dic['links'].append(dic[link])
            del dic[link]

        return dic

    def to_json(self):
        dic = self._get_dict()
        dic = self._enhance_dic(dic)
        return pretty_json(dic)

    def to_xml(self):
        dic = self._get_dict()
        dic = self._enhance_dic(dic)
        xml_str = dicttoxml.dicttoxml(dic, custom_root=self.__class__.__name__, attr_type=False)
        xml_str = parseString(xml_str).toprettyxml()
        return xml_str

    def to_dict(self):
        return self._get_dict()

    def to_jsonld(self):
        dic = self._get_dict()
        dic['@id'] = self.resource_path
        dic['@context'] = self.__contexts__ + self._contexts
        dic['@type'] = self.types
        #dic['additionalType'] = self._types
        dic['links'] = []

        for link in self._links:
            dic['links'].append(dic[link])
            del dic[link]

        return pretty_json(dic)

    def to_html(self):
        return ''

    def _create_path(self, parts, prefix=''):
        new_parts = [prefix]
        for part in parts:
            part = str(part)
            if part[0] == '/':
                part = part[1:]
            if part[-1] == '/':
                part = part[0:-1]
            new_parts.append(part)
        return "/".join(new_parts)


class Resource(Entity):
    __type__ = 'xwot:Resource'


class CollectionMember(Resource):
    __type__ = ['xwot:Resource']

    @property
    def id(self):
        raise NotImplementedError


class Collection(Resource):
    __type__ = ['xwot:Resource', 'hydra:Collection']

    @property
    def members(self):
        raise NotImplementedError

    def _enhance_dic(self, dic):
        dic['type'] = self.__type__
        #dic['additionalType'] = self._types
        dic['members'] = []

        for member in self.members:
            item_link = member.resource_path
            item_dic = {
                'type': member.type,
                'link': item_link
            }
            dic['members'].append(item_dic)

        dic['links'] = []
        for link in self._links:
            dic['links'].append(dic[link])
            del dic[link]

        return dic

    def to_jsonld(self):
        dic = self._get_dict()
        dic['@id'] = self.resource_path
        dic['@context'] = self.__contexts__
        dic['@type'] = self.types

        #dic['additionalType'] = self._types
        dic['members'] = []

        for member in self.members:
            item_link = member.resource_path
            item_dic = {
                '@type': member.types,
                '@id': item_link
            }
            dic['members'].append(item_dic)

        dic['links'] = []

        for link in self._links:
            dic['links'].append(dic[link])
            del dic[link]

        return pretty_json(dic)


class Device(Entity):
    __type__ = 'xwot:Device'


class Sensor(Entity):
    __type__ = 'xwot:Sensor'


class Actuator(Entity):
    __type__ = 'xwot:Actuator'


class Context(Entity):
    __type__ = 'xwot:Context'


class Tag(Entity):
    __type__ = 'xwot:Tag'


class Service(Entity):
    __type__ = 'xwot:Service'


class Publisher(Entity):
    __type__ = 'xwot:Publisher'


class Model(object):

    __mutable_props__ = []

    def update(self, dic, content_type):
        _list = content_type.split(';') # parse content type headers e.g.: application/json;charset=UTF-8
        _content_type = _list[0]

        if _content_type in ['application/json', 'application/ld+json']:
            return self.handle_update(dic)
        elif _content_type in ['application/xml']:
            dic = dic.get(self.__class__.__name__, {})
            return self.handle_update(dic)
        else:
            return 400

    def handle_update(self, dic):
        raise NotImplementedError


class BaseModel(Model):

    __mutable_props__ = []

    def __init__(self, dic=None):
        self._dic = dic or {}

    def handle_update(self, dic):
        valid_keys = [key for key in dic.keys() if key in self.__mutable_props__]
        for key in valid_keys:
            self._dic[key] = str(dic[key])

        return 200




import sqlite3

class SubscriberDB():

    @staticmethod
    def getAllClients(resource='1'):
        conn = sqlite3.connect('clients.db')
        c = conn.cursor()
        c.execute("Select * from Subscriber where resourceid='"+resource+"' order by id")
        result = c.fetchall()
        c.close()
        conn.close()
        return result

    @staticmethod
    def insertClient(uri, method, accept, resource=1):
        conn = sqlite3.connect('clients.db')
        c = conn.cursor()
        c.execute("insert into Subscriber (uri, method, accept, resourceid) values ('"+uri+"', '"+method+"', '"+accept+"', '"+resource+"')")
        result = c.lastrowid
        conn.commit()
        c.close()
        conn.close()
        return result

    @staticmethod
    def updateClient(uri, method, accept, clientid):
        conn = sqlite3.connect('clients.db')
        c = conn.cursor()
        c.execute("Update Subscriber set uri='"+uri+"', method='"+method+"', accept='"+accept+"' where id='"+clientid+"'")
        result = c.lastrowid
        conn.commit()
        c.close()
        conn.close()
        return result

    @staticmethod
    def deleteQuery(clientid):
        conn = sqlite3.connect('clients.db')
        c = conn.cursor()
        queryresult = c.execute("Delete from Subscriber where id='"+clientid+"'")
        conn.commit()
        c.close()
        conn.close()
        return queryresult

    @staticmethod
    def getClient(clientid):
        conn = sqlite3.connect('clients.db')
        c = conn.cursor()
        c.execute("Select * from Subscriber where id='"+clientid+"'")
        result = c.fetchall()[0]
        c.close()
        conn.close()
        return result



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# xwot.py - Python tools for the extended Web of Things
# Copyright (C) 2015  Alexander Rüedlinger
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

__author__ = 'Alexander Rüedlinger'

from helper import OutputPrinter
from xwot.compiler.frontend import visitor


class ExpressBackendBuilder(visitor.BaseVisitor):

    def __init__(self):
        visitor.BaseVisitor.__init__(self)
        self._out = OutputPrinter()

    def before(self):
        self._out.code([
            '/*',
            ' * Generated by xwot compiler.',
            ' *',
            ' * Express.js xwot application.',
            ' */',
            '',
            'module.exports = function() {',
            self._out.indent([
                "var express = require('express');",
                'var app = express();',
            ])
        ], ["xwot", "app"])

    def after(self):
        self._out.code([
            self._out.indent(
                self._out.flatten([
                    [
                        "var %s = require('./%s.js');" % (node.name(), node.name()) for node in self._nodes
                    ],
                    [
                        "",
                        "//register resources"
                    ],
                    [
                        "%s(app);" % node.name() for node in self._nodes
                    ],
                    [
                        '',
                        'return app;'
                    ]
                ])
        ),
        "};"
        ], ["xwot", "app"])

        self._out.code([
            '/*',
            ' * Generated by xwot compiler.',
            ' *',
            ' * Express.js xwot application.',
            ' */',
            '',
            "var xwot_app = require('./xwot/app.js')();",
            '',
            "var server = xwot_app.listen(3000, function() {",
            self._out.indent([
                'var host = server.address().address;',
                'var port = server.address().port;',
                "console.log('Example app listening at http://%s:%s', host, port);"
            ]),
            '});'
        ], ["runserver"])

    def before_resource(self, node):
        self._out.code([
            '/*',
            ' * Generated by xwot compiler.',
            ' *',
            " * Type:       %s" % node.type(),
            " * Resource:   %s" % node.name(),
            " * Path:       %s" % node.fullpath(),
            '*/',
            '',
            'var setup = function(app) {',
        ], ["xwot", node.name()])

    def after_resource(self, node):
        self._out.code([
            '}',
            '',
            'module.exports = setup;'
        ], ["xwot", node.name()])

    def handle_entity(self, node):
        self.before_resource(node)
        self._out.code([
            self._out.indent([
                '/*',
                 " * GET '/'",
                 ' */',
                "app.get('/', function(req, res) { ",
                self._out.indent([
                    "res.send('Hello World at: %s');" % node.fullpath()
                ]),
                '});'
            ]),
        ], ["xwot", node.name()])
        self.after_resource(node)

    def handle_resource(self, node):
        for method in ['get', 'put']:
            self._out.code([
                self._out.indent([
                    '/*',
                    " * %s '%s'" % (method.upper(), node.fullpath()),
                    ' */',
                    "app.%s('%s', function(req, res) { " % (method, node.fullpath()),
                    self._out.indent([
                            'res.end("Hello World at: %s");' % node.fullpath()
                        ]),
                    '});'
                ]),
            ], ["xwot", node.name()])

    def handle_device_resource(self, node):
        for method in ['get', 'put']:
            self._out.code([
                self._out.indent([
                    '/*',
                    " * %s '%s'" % (method.upper(), node.fullpath()),
                    ' */',
                    "app.%s('%s', function(req, res) { " % (method, node.fullpath()),
                    self._out.indent([
                            'res.end("Hello World at: %s");' % node.fullpath()
                        ]),
                    '});'
                ]),
            ], ["xwot", node.name()])

    def handle_sensor_resource(self, node):
        for method in ['get']:
            self._out.code([
                self._out.indent([
                    '/*',
                    " * %s '%s'" % (method.upper(), node.fullpath()),
                    ' */',
                    "app.%s('%s', function(req, res) { " % (method, node.fullpath()),
                    self._out.indent([
                            'res.end("Hello World at: %s");' % node.fullpath()
                        ]),
                    '});'
                ]),
            ], ["xwot", node.name()])

    def handle_tag_resource(self, node):
        for method in ['get']:
            self._out.code([
                self._out.indent([
                    '/*',
                    " * %s '%s'" % (method.upper(), node.fullpath()),
                    ' */',
                    "app.%s('%s', function(req, res) { " % (method, node.fullpath()),
                    self._out.indent([
                            'res.end("Hello World at: %s");' % node.fullpath()
                        ]),
                    '});'
                ]),
            ], ["xwot", node.name()])

    def handle_context_resource(self, node):
        for method in ['get', 'put']:
            self._out.code([
                self._out.indent([
                    '/*',
                    " * %s '%s'" % (method.upper(), node.fullpath()),
                    ' */',
                    "app.%s('%s', function(req, res) { " % (method, node.fullpath()),
                    self._out.indent([
                            'res.end("Hello World at: %s");' % node.fullpath()
                        ]),
                    '});'
                ]),
            ], ["xwot", node.name()])

    def handle_service_resource(self, node):
        for method in ['get', 'post', 'put', 'delete']:
            self._out.code([
                self._out.indent([
                    '/*',
                    " * %s '%s'" % (method.upper(), node.fullpath()),
                    ' */',
                    "app.%s('%s', function(req, res) { " % (method, node.fullpath()),
                    self._out.indent([
                            'res.end("Hello World at: %s");' % node.fullpath()
                        ]),
                    '});'
                ]),
            ], ["xwot", node.name()])

    def handle_actuator_resource(self, node):
        for method in ['put']:
            self._out.code([
                self._out.indent([
                    '/*',
                    " * %s '%s'" % (method.upper(), node.fullpath()),
                    ' */',
                    "app.%s('%s', function(req, res) { " % (method, node.fullpath()),
                    self._out.indent([
                            'res.end("Hello World at: %s");' % node.fullpath()
                        ]),
                    '});'
                ]),
            ], ["xwot", node.name()])

    def handle_publisher_resource(self, node):
        for method in ['get', 'post']:
            self._out.code([
                self._out.indent([
                    '/*',
                    " * %s '%s'" % (method.upper(), node.fullpath()),
                    ' */',
                    "app.%s('%s', function(req, res) { " % (method, node.fullpath()),
                    self._out.indent([
                            'res.end("Hello World at: %s");' % node.fullpath()
                        ]),
                    '});'
                ]),
            ], ["xwot", node.name()])

    def output(self):
        # get all created files
        files = self._out.flush()
        new_files_dic = {}

        for file_name, code in files.items():
            # append to each file its correct file extension
            new_files_dic[file_name + ".js"] = code

        return new_files_dic


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# xwot.py - Python tools for the extended Web of Things
# Copyright (C) 2015  Alexander Rüedlinger
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

__author__ = 'Alexander Rüedlinger'

from xwot.model import Context as XWOTContext
from xwot.model import Device as XWOTDevice
from xwot.model import Model
from xwot.model import BaseModel

from xwot.i2c.adapter import WindowAdapter
from xwot.i2c.adapter import ShutterAdapter
from xwot.device.window import Handle as WindowHandle


class Window(XWOTDevice, BaseModel):

    __mutable_props__ = ['name', 'streetAddress', 'roomAddress', 'postalCode', 'addressLocality']
    __expose__ = __mutable_props__ + ['description', 'handle', 'shutter']

    def __init__(self, name, street_address, postal_code, address_locality, room_address):
        super(Window, self).__init__()
        self._dic = {
            'name': name,
            'streetAddress': street_address,
            'postalCode': postal_code,
            'addressLocality': address_locality,
            'roomAddress': room_address
        }

        self.add_type('xwot-ext:Window')
        self.add_link('handle')
        self.add_link('shutter')

    @property
    def resource_path(self):
        return '/window'

    @property
    def name(self):
        return self._dic['name']

    @property
    def description(self):
        return "Hi there my name is %s. I'm a window and currently present in room %s at the location: %s, %s, %s" % \
               (self.name, self.roomAddress, self.streetAddress, self.addressLocality, self.postalCode)

    @property
    def handle(self):
        return '/window/handle'

    @property
    def shutter(self):
        return '/window/shutter'

    @property
    def streetAddress(self):
        return self._dic['streetAddress']

    @property
    def postalCode(self):
        return self._dic['postalCode']

    @property
    def addressLocality(self):
        return self._dic['addressLocality']

    @property
    def roomAddress(self):
        return self._dic['roomAddress']


class Handle(WindowHandle):

    def __init__(self, name, adapter=WindowAdapter()):
        super(Handle, self).__init__(name=name, adapter=adapter)


class Shutter(XWOTContext, Model):
    __mutable_props__ = ['name', 'state']
    __expose__ = __mutable_props__ + ['description', 'window']

    def __init__(self, name, adapter=ShutterAdapter()):
        super(Shutter, self).__init__()

        self._dic = {
            'name': name
        }
        self._adapter = adapter
        self.add_type('xwot-ext:Shutter')
        self.add_link('window')

    @property
    def resource_path(self):
        return '/window/shutter'

    @property
    def window(self):
        return '/window'

    @property
    def description(self):
        return "A shutter window which can be moved up and moved down."

    @property
    def state(self):
        return self._adapter.state

    @property
    def name(self):
        return self._dic['name']

    def handle_update(self, dic):
        if dic.get('state') == 'down':
            self._adapter.down()

        if dic.get('state') == 'up':
            self._adapter.up()

        self._dic['name'] = str(dic.get('name', self._dic['name']))

        return 200


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# xwot.py - Python tools for the extended Web of Things
# Copyright (C) 2015  Alexander Rüedlinger
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

__author__ = 'Alexander Rüedlinger'

from xwot.model import Model
from xwot.model import BaseModel
from xwot.model import Device as XWOTDevice
from xwot.model import Sensor as XWOTSensor
from xwot.model import Collection
from xwot.model import CollectionMember

__all__ = ['WeatherStation', 'GPSWeatherStation', 'SensorCollection', 'GPS']


class WeatherStation(XWOTDevice, BaseModel):
    __mutable_props__ = ['name', 'streetAddress', 'roomAddress', 'postalCode', 'addressLocality']
    __expose__ = __mutable_props__ + ['description', 'sensors']

    def __init__(self, name, street_address, postal_code, address_locality, room_address):
        super(WeatherStation, self).__init__()
        self._dic = {
            'name': name,
            'streetAddress': street_address,
            'postalCode': postal_code,
            'addressLocality': address_locality,
            'roomAddress': room_address
        }

        self.add_type('xwot-ext:WeatherStation')
        self.add_link('sensors')

    @property
    def resource_path(self):
        return '/weatherstation'

    @property
    def name(self):
        return self._dic['name']

    @property
    def description(self):
        return "Hi there my name is %s. I'm a weather station and currently present in room %s at the location: %s, %s, %s" % \
               (self.name, self.roomAddress, self.streetAddress, self.addressLocality, self.postalCode)

    @property
    def sensors(self):
        return '/weatherstation/sensors'

    @property
    def streetAddress(self):
        return self._dic['streetAddress']

    @property
    def postalCode(self):
        return self._dic['postalCode']

    @property
    def addressLocality(self):
        return self._dic['addressLocality']

    @property
    def roomAddress(self):
        return self._dic['roomAddress']

    @streetAddress.setter
    def streetAddress(self, v):
        self._dic['streetAddress'] = v

    @postalCode.setter
    def postalCode(self, v):
        self._dic['postalCode'] = v

    @addressLocality.setter
    def addressLocality(self, v):
        self._dic['addressLocality'] = v

    @roomAddress.setter
    def roomAddress(self, v):
        self._dic['roomAddress'] = v

    @property
    def sensors(self):
        return '/weatherstation/sensors'


class GPSWeatherStation(WeatherStation):

    __mutable_props__ = ['name', 'roomAddress']
    __expose__ = __mutable_props__ + ['description', 'sensors', 'gps', 'streetAddress', 'roomAddress', 'postalCode',
                                      'addressLocality']

    def __init__(self, name='GPS Weather Station', room_address='Unknown'):
        super(GPSWeatherStation, self).__init__(name=name, room_address=room_address, street_address='',
                                                postal_code='', address_locality='')
        self.add_link('gps')

    @property
    def gps(self):
        return '/weatherstation/gps'


class Sensor(XWOTSensor, CollectionMember, Model):
    __expose__ = ['name', 'unit', 'measures', 'description', 'measurement', 'symbol', 'waterdispenser',
                  'back_link1', 'back_link2']

    def __init__(self, name, unit, symbol, description, measures, type_iri, adapter, adapter_measurement_fun, id):
        super(Sensor, self).__init__()
        self._name = name
        self._id = id
        self._unit = unit
        self._symbol = symbol
        self._description = description
        self._measures = measures
        self._adapter = adapter
        self.add_type(type_iri)
        self._adapter_measurement_fun = adapter_measurement_fun
        self.add_link('back_link1')
        self.add_link('back_link2')

    @property
    def resource_path(self):
        return "/weatherstation/sensors/%s" % self._id

    @property
    def id(self):
        return self._id

    @property
    def back_link1(self):
        return '/weatherstation/sensors'

    @property
    def back_link2(self):
        return '/weatherstation'

    @property
    def name(self):
        return self._name

    @property
    def unit(self):
        return self._unit

    @property
    def symbol(self):
        return self._symbol

    @property
    def description(self):
        return self._description

    @property
    def measures(self):
        return self._measures

    @property
    def measurement(self):
        val = self._adapter_measurement_fun(self._adapter)
        if val is not None:
            return round(val, 2)
        else:
            return val

    def handle_update(self, dic):
        pass


from xwot.i2c.adapter import WeatherstationAdapter


def create_sensors():
    adapter = WeatherstationAdapter()

    _sensors = {
        'temperature_1': Sensor(id='temperature1', name='Temperature sensor 1', unit='Celsius', symbol='°C',
                                measures='Temperature',
                                description='A temperature sensor of this weather station.',
                                adapter=adapter, adapter_measurement_fun=lambda a: a.temperature_1,
                                type_iri='xwot-ext:TemperatureSensor'),

        'temperature_2': Sensor(id='temperature2', name='Temperature sensor 2', unit='Celsius', symbol='°C',
                                measures='Temperature',
                                description='A temperature sensor of this weather station.',
                                adapter=adapter, adapter_measurement_fun=lambda a: a.temperature_2,
                                type_iri='xwot-ext:TemperatureSensor'),

        'pressure': Sensor(id='pressure', name='Pressure sensor', unit='Pascal', symbol='pa', measures='Pressure',
                           description='A pressure sensor of this weather station.',
                           adapter=adapter, adapter_measurement_fun=lambda a: a.pressure,
                           type_iri='xwot-ext:PressureSensor'),

        'humidity': Sensor(id='humidity', name='Humidity sensor', unit='Percentage', symbol='%',
                           measures='Humidity',
                           description='A humidity sensor of this weather station.',
                           adapter=adapter, adapter_measurement_fun=lambda a: a.humidity,
                           type_iri='xwot-ext:HumiditySensor'),

        'altitude': Sensor(id='altitude', name='Altitude sensor', unit='Meters', symbol='m', measures='Altitude',
                           description='An altitude sensor of this weather station.',
                           adapter=adapter, adapter_measurement_fun=lambda a: a.altitude,
                           type_iri='xwot-ext:AltitudeSensor'),

        'illuminance': Sensor(id='illuminance', name='Illuminance sensor', unit='Lux', symbol='lx',
                              measures='Illuminance',
                              description='An illuminance sensor of this weather station.',
                              adapter=adapter, adapter_measurement_fun=lambda a: a.illuminance,
                              type_iri='xwot-ext:IlluminanceSensor'),

        'color': Sensor(id='color', name='Color sensor', unit='Kelvin', symbol='k', measures='Temperature',
                        description='A color sensor of this weather station.',
                        adapter=adapter, adapter_measurement_fun=lambda a: a.color_temperature,
                        type_iri='xwot-ext:ColorSensor')
    }

    return _sensors


class SensorCollection(Collection, Model):

    __expose__ = ['back_link']

    def __init__(self, sensors):
        super(SensorCollection, self).__init__()
        self._sensors = sensors
        self.add_link('back_link')

    @property
    def resource_path(self):
        return '/weatherstation/sensors'

    @property
    def members(self):
        return self._sensors

    @property
    def back_link(self):
        return '/weatherstation'

    def handle_update(self, dic):
        pass


from xwot.i2c.adapter import GPSAdapter


class GPS(XWOTDevice, Model):

    __expose__ = ['name', 'description', 'state', 'latitude', 'longitude', 'elevation', 'back_link']

    def __init__(self):
        super(GPS, self).__init__()
        self._adapter = GPSAdapter()
        self.add_type('schema:GeoCoordinates')
        self.add_link('back_link')

    @property
    def resource_path(self):
        return '/weatherstation/gps'

    @property
    def state(self):
        if self._adapter.found:
            return 'found'
        else:
            return 'lost'

    @property
    def found(self):
        return self._adapter.found

    @property
    def latitude(self):
        return self._adapter.latitude

    @property
    def longitude(self):
        return self._adapter.longitude

    @property
    def elevation(self):
        return self._adapter.elevation

    @property
    def back_link(self):
        return '/weatherstation'

    @property
    def description(self):
        return "A GPS navigation device is a device that accurately calculates geographical location " \
               "by receiving information from GPS satellites."

    @property
    def name(self):
        return 'GPS device'

    def handle_update(self, dic):
        pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# xwot.py - Python tools for the extended Web of Things
# Copyright (C) 2015  Alexander Rüedlinger
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

__author__ = 'Alexander Rüedlinger'
import xml.etree.ElementTree as ET

import node


class Parser(object):

    def parse(self, xwot_file):
        tree = ET.parse(xwot_file)
        root = tree.getroot()
        parse_tree = TreeWalker.walk(root)
        return parse_tree

    def parse_str(self, xml):
        tree = ET.fromstring(xml)
        root = tree.getroot()
        parse_tree = TreeWalker.walk(root)
        return parse_tree


class TreeWalker(object):

    class NodeHelper:

        # parsing actions
        @classmethod
        def create_vnode(cls, clazz):
            """
            Creates a virtual entity node.
            :param clazz:
            :return:
            """
            def create_vnode(node, dict_attr):
                uri = dict_attr['uri']
                name = dict_attr['name']
                return clazz(name, uri, dict_attr)

            return create_vnode


        @classmethod
        def create_pnode(cls, clazz):
            """
            Creates a physical entity node.
            :param clazz:
            :return:
            """
            def create_node(node, dict_attr):
                if 'name' in dict_attr:
                    name = dict_attr['name']
                else:
                    name = 'RootResource'
                return clazz(name, dict_attr)

            return create_node


    # mapped namespaces / namespace lookup table
    NAMESPACE = {
        'http://www.omg.org/XMI': 'xmi',
        'http://www.w3.org/2001/XMLSchema-instance': 'xsi',
        'http://diuf.unifr.ch/softeng': 'xwot'
    }



    # parsing action table
    ACTION = {
        'xwot:ActuatorResource': NodeHelper.create_vnode(node.ActuatorResource),
        'xwot:SensorResource': NodeHelper.create_vnode(node.SensorResource),
        'xwot:DeviceResource': NodeHelper.create_vnode(node.DeviceResource),
        'xwot:ContextResource': NodeHelper.create_vnode(node.ContextResource),
        'xwot:TagResource': NodeHelper.create_vnode(node.TagResource),
        'xwot:ServiceResource': NodeHelper.create_vnode(node.ServiceResource),
        'xwot:Resource': NodeHelper.create_vnode(node.Resource),
        'xwot:PublisherResource': NodeHelper.create_vnode(node.PublisherResource),

        'xwot:Actuator': NodeHelper.create_pnode(node.Actuator),
        'xwot:Sensor': NodeHelper.create_pnode(node.Sensor),
        'xwot:Device': NodeHelper.create_pnode(node.Device),
        'xwot:Tag': NodeHelper.create_pnode(node.Tag),

        'xwot:Entity': NodeHelper.create_pnode(node.Entity)
    }


    @classmethod
    def _resolve_namespace(cls, uri):
        """
        Resolves the uri to a namespace.
        :param uri:
        :return:
        """
        return TreeWalker.NAMESPACE[uri] if uri in TreeWalker.NAMESPACE else uri


    @classmethod
    def _parse_ns(cls, tag):
        """
        Parses the ns prefix of a tag or attr key.
        :param tag:
        :return:
        """
        if tag[0] == '{':
            index = tag.index('}')
            ns = tag[1:index]
            name = tag[index+1:]
            return TreeWalker._resolve_namespace(ns) + ':' + name
        return tag


    @classmethod
    def _parse_attrs(cls, node):
        """
        Parse attributes of a node and returns a hash with resolved namespaces.
        :param node:
        :return:
        """
        pairs = map(lambda (key, value): (TreeWalker._parse_ns(key), value), node.attrib.items())
        return dict(pairs)


    @classmethod
    def walk(cls, node):
        root_node = TreeWalker._create_node(node)
        for child in node:
            child_node = TreeWalker.walk(child)
            root_node.add_child(child_node)

        return root_node


    @classmethod
    def _create_node(cls, node):
        dict_attr = TreeWalker._parse_attrs(node)

        if 'xsi:type' in dict_attr:
            node_type = dict_attr['xsi:type']
        else:
            node_type = TreeWalker._parse_ns(node.tag)
        parse_action = TreeWalker.ACTION[node_type]

        return parse_action(node, dict_attr)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# xwot.py - Python tools for the extended Web of Things
# Copyright (C) 2015  Alexander Rüedlinger
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

__author__ = 'Alexander Rüedlinger'

import json
from visitor import Visitor
from parser import Parser


class TreeCleaner(Visitor):
    """
    TreeCleaner removes all physical nodes and adds absolute uri to all nodes.
    """

    def __init__(self):
        self._path = ['']

    def path(self):
        return "/".join(self._path)

    def visit_entity(self, node):
        virtual_children = [child for child in node.children() if child.is_virtual()]
        [child.accept(self) for child in virtual_children]
        node.set_children(virtual_children)

    def visit_resource(self, node):
        fullpath = self.path() + '/' + node.uri()
        node._fullpath = fullpath
        self._path.append(node.uri())
        for child in node.children():
            child.accept(self)

        self._path.pop()

    def visit_device_resource(self, node):
        fullpath = self.path() + '/' + node.uri()
        node._fullpath = fullpath

        self._path.append(node.uri())
        for child in node.children():
            child.accept(self)

        self._path.pop()

    def visit_sensor_resource(self, node):
        fullpath = self.path() + '/' + node.uri()
        node._fullpath = fullpath

        self._path.append(node.uri())
        for child in node.children():
            child.accept(self)

        self._path.pop()

    def visit_service_resource(self, node):
        fullpath = self.path() + '/' + node.uri()
        node._fullpath = fullpath

    def visit_actuator_resource(self, node):
        fullpath = self.path() + '/' + node.uri()
        node._fullpath = fullpath

        self._path.append(node.uri())
        for child in node.children():
            child.accept(self)

        self._path.pop()

    def visit_tag_resource(self, node):
        fullpath = self.path() + '/' + node.uri()
        node._fullpath = fullpath

        self._path.append(node.uri())
        for child in node.children():
            child.accept(self)

        self._path.pop()

    def visit_context_resource(self, node):
        fullpath = self.path() + '/' + node.uri()
        node._fullpath = fullpath

        self._path.append(node.uri())
        for child in node.children():
            child.accept(self)

        self._path.pop()

    def visit_publisher_resource(self, node):
        fullpath = self.path() + '/' + node.uri()
        node._fullpath = fullpath

    def visit_device(self, node):
        pass

    def visit_sensor(self, node):
        pass

    def visit_actuator(self, node):
        pass

    def visit_tag(self, node):
        pass


class JSONLDDescriptionPrinter(Visitor):
    """
    JSONLDDescriptionPrinter creates a minimal device description file in the jsonld format.
    """

    CHILDREN = 'knows'
    TYPE = "@type"
    ID = "@id"
    CONTEXT = '@context'

    def __init__(self, site='', base=None):
        self._output = None
        self._current = None
        self._site = self._remove_last_forwardslash(site)
        self._base = base
        self._path = [self._site]

    def _remove_last_forwardslash(self, site):
        _site = site
        if len(site) > 0:
            if site[-1] == '/':
                _site = site[0:-1]

        return _site

    @property
    def site(self):
        return self._site

    @property
    def base(self):
        return self._base

    @base.setter
    def base(self, value):
        self._base = value

    @site.setter
    def site(self, value):
        self._site = self._remove_last_forwardslash(value)
        self._path[0] = self._site

    def path(self):
        return "/".join(self._path)

    def visit_entity(self, node):
        self._output = {
            ("%s" % self.ID): '/',
            ("%s" % self.TYPE): 'xwot:Description',
            ("%s" % self.CHILDREN): []
        }

        before = self._output
        self._current = self._output[("%s" % self.CHILDREN)]

        [child.accept(self) for child in node.children() if child.is_virtual()]

        # overwrite entity resource and use the first child as root
        # entity resource has always at most one child !!!
        #self._output = self._output[("%s" % self.CHILDREN)][0]  # hack
        self._output[self.CONTEXT] = ["http://xwot.lexruee.ch/contexts/xwot"]

    def visit_resource(self, node):
        resource = {
            ("%s" % self.ID): self.path() + '/' + node.uri(),
            'uri': {
                "@id": self.path() + '/' + node.uri()
            },
            'label': node.name(),
            'sameAs': "http://www.productontology.org/id/Web_resource",
            'additionalType': {
                '@id': "http://xwot.lexruee.ch/vocab/core#Resource"
            },
            ("%s" % self.TYPE): 'xwot:Resource',
            ("%s" % self.CHILDREN): []
        }

        self._current.append(resource)
        before = self._current
        self._current = resource[("%s" % self.CHILDREN)]
        self._path.append(node.uri())
        for child in node.children():
            child.accept(self)

        self._current = before
        self._path.pop()

    def visit_device_resource(self, node):
        device = {
            ("%s" % self.ID): self.path() + '/' + node.uri(),
            'uri': {
                "@id": self.path() + '/' + node.uri()
            },
            'label': node.name(),
            'sameAs': "http://www.productontology.org/id/Computer_appliance",
            'additionalType': {
                '@id': "http://xwot.lexruee.ch/vocab/core#Device"
            },
            ("%s" % self.TYPE): 'xwot:Device',
            ("%s" % self.CHILDREN): []
        }

        self._current.append(device)
        before = self._current
        self._current = device[("%s" % self.CHILDREN)]

        self._path.append(node.uri())
        for child in node.children():
            child.accept(self)

        self._current = before
        self._path.pop()

    def visit_sensor_resource(self, node):
        sensor = {
            ("%s" % self.ID): self.path() + '/' + node.uri(),
            'uri': {
                "@id": self.path() + '/' + node.uri()
            },
            'label': node.name(),
            'sameAs': "http://www.productontology.org/id/Sensor",
            'additionalType': {
                '@id': "http://xwot.lexruee.ch/vocab/core#Sensor"
            },
            ("%s" % self.TYPE): 'xwot:Sensor',
            ("%s" % self.CHILDREN): []
        }
        self._current.append(sensor)

        before = self._current
        self._current = sensor[("%s" % self.CHILDREN)]

        self._path.append(node.uri())
        for child in node.children():
            child.accept(self)

        self._current = before
        self._path.pop()

    def visit_service_resource(self, node):
        service = {
            ("%s" % self.ID): self.path() + '/' + node.uri(),
            'uri': {
                "@id": self.path() + '/' + node.uri()
            },
            'label': node.name(),
            'sameAs': "http://www.productontology.org/id/Service",
            'additionalType': {
                '@id': "http://xwot.lexruee.ch/vocab/core#Service"
            },
            ("%s" % self.TYPE): 'xwot:Service'
        }
        self._current.append(service)

    def visit_actuator_resource(self, node):
        actuator = {
            ("%s" % self.ID): self.path() + '/' + node.uri(),
            'uri': {
                "@id": self.path() + '/' + node.uri()
            },
            'label': node.name(),
            ("%s" % self.TYPE): 'xwot:Actuator',
            'sameAs': "http://www.productontology.org/id/Actuator",
            'additionalType': {
                '@id': "http://xwot.lexruee.ch/vocab/core#Actuator"
            },
            ("%s" % self.CHILDREN): []
        }
        self._current.append(actuator)

        before = self._current
        self._current = actuator[("%s" % self.CHILDREN)]

        self._path.append(node.uri())
        for child in node.children():
            child.accept(self)

        self._current = before
        self._path.pop()

    def visit_tag_resource(self, node):
        tag = {
            ("%s" % self.ID): self.path() + '/' + node.uri(),
            'uri': {
                "@id": self.path() + '/' + node.uri()
            },
            'label': node.name(),
            'sameAs': "http://www.productontology.org/id/Automatic_identification_and_data_capture",
            'additionalType': {
                '@id': "http://xwot.lexruee.ch/vocab/core#Tag"
            },
            ("%s" % self.TYPE): 'xwot:Tag',
            ("%s" % self.CHILDREN): []
        }
        self._current.append(tag)

        before = self._current
        self._current = tag[("%s" % self.CHILDREN)]

        self._path.append(node.uri())
        for child in node.children():
            child.accept(self)

        self._current = before
        self._path.pop()

    def visit_context_resource(self, node):
        context = {
            ("%s" % self.ID): self.path() + '/' + node.uri(),
            'uri': {
                "@id": self.path() + '/' + node.uri()
            },
            'label': node.name(),
            'sameAs': "http://www.productontology.org/id/Context",
            'additionalType': {
                '@id': "http://xwot.lexruee.ch/vocab/core#Context"
            },
            ("%s" % self.TYPE): 'xwot:Context',
            ("%s" % self.CHILDREN): []
        }
        self._current.append(context)

        before = self._current
        self._current = context[("%s" % self.CHILDREN)]

        self._path.append(node.uri())
        for child in node.children():
            child.accept(self)

        self._current = before
        self._path.pop()

    def visit_publisher_resource(self, node):
        publisher = {
            ("%s" % self.ID): self.path() + '/' + node.uri(),
            'uri': {
                "@id": self.path() + '/' + node.uri()
            },
            'label': node.name(),
            'sameAs': "http://www.productontology.org/id/Publish%E2%80%93subscribe_pattern",
            'additionalType': {
                '@id': "http://xwot.lexruee.ch/vocab/core#Publisher"
            },
            ("%s" % self.TYPE): 'xwot:Publisher',
            ("%s" % self.CHILDREN): []
        }
        self._current.append(publisher)

    def visit_device(self, node):
        device = {
            'label': node.name(),
            ("%s" % self.TYPE): 'xwot:Device',
            ("%s" % self.CHILDREN): []
        }

        self._current.append(device)
        before = self._current
        self._current = device[("%s" % self.CHILDREN)]

        for child in node.children():
            child.accept(self)

        self._current = before

    def visit_sensor(self, node):
        sensor = {
            'label': node.name(),
            ("%s" % self.TYPE): 'xwot:Sensor'
        }
        self._current.append(sensor)

    def visit_actuator(self, node):
        actuator = {
            'label': node.name(),
            ("%s" % self.TYPE): 'xwot:Actuator'
        }
        self._current.append(actuator)

    def visit_tag(self, node):
        tag = {
            'label': node.name(),
            ("%s" % self.TYPE): 'xwot:Tag'
        }
        self._current.append(tag)

    def output(self):
        if self._base is not None:
            self._output[self.CONTEXT].append({
                "@base": self._base
            })

        out = json.dumps(self._output, indent=4, sort_keys=True, separators=(',', ': '))
        return out


class DescriptionBuilder(object):

    def __init__(self, description_printer):
        self._description_printer = description_printer
        self._out = None

    def build(self, xml_file, site='', base=None):
        parser = Parser()
        root_node = parser.parse(xml_file)

        # set properties for creating absolute paths
        self._description_printer.site = site  # uses the site path as prefix for all relative uris
        self._description_printer.base = base  # sets the jsonld @base property in @context

        root_node.accept(self._description_printer)
        self._out = self._description_printer.output()
        return self._out

    def output(self):
        return self._out


class JSONLDDescriptionBuilder(DescriptionBuilder):

    def __init__(self):
        super(JSONLDDescriptionBuilder, self).__init__(JSONLDDescriptionPrinter())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# xwot.py - Python tools for the extended Web of Things
# Copyright (C) 2015  Alexander Rüedlinger
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

__author__ = 'Alexander Rüedlinger'

from xwot.util.vocab import NullIri


class Resource(object):
    def __init__(self, cls, description=None, iri=None, route_keys=None):
        self._cls = cls
        self._properties = {}
        self._description = description or ''
        self._route_keys = route_keys or []
        self._iri = iri or NullIri()

    @property
    def description(self):
        return self._description

    @property
    def iri(self):
        return self._iri

    @property
    def name(self):
        return self._cls.__name__

    @property
    def route_keys(self):
        return self._route_keys

    @property
    def properties(self):
        return self._properties

    @properties.setter
    def properties(self, value):
        self._properties = value


class Property(object):
    def __init__(self, name, iri, description, route_keys, required, readonly,
                 writeonly, range):
        self._name = name
        self._range = range
        self._description = description
        self._iri = iri or NullIri()
        self._route_keys = route_keys or []
        self._required = required
        self._readonly = readonly
        self._writeonly = writeonly

    @property
    def iri(self):
        return self._iri

    @property
    def range(self):
        return self._range

    @property
    def name(self):
        return self._name

    @property
    def writeonly(self):
        return self._writeonly

    @property
    def readonly(self):
        return self._readonly

    @property
    def required(self):
        return self._required

    @property
    def route_keys(self):
        return self._route_keys

    @property
    def description(self):
        return self._description

    @property
    def description(self):
        return self._description


class Route(object):
    def __init__(self, name, method, description=None, returns=None, status_codes=None, expects=None):
        self._name = name
        self._method = method
        self._description = description


        if status_codes is None:
            self._status_codes = []
        else:
            self._status_codes = status_codes

        self._expects = expects or NullIri()
        self._returns = returns or NullIri()

    @property
    def name(self):
        return self._name

    @property
    def method(self):
        return self._method

    @property
    def description(self):
        return self._description

    @property
    def returns(self):
        return self._returns

    @property
    def status_codes(self):
        return self._status_codes

    @property
    def expects(self):
        return self._expects


class Documentation(object):
    def __init__(self, apidoc_url='', title=None, description=None, entrypoint=None):
        self._title = title
        self._description = description
        self._apidoc_url = apidoc_url
        self._entrypoint = entrypoint

    @property
    def title(self):
        return self._title

    @property
    def apidoc_url(self):
        return self._apidoc_url

    @property
    def description(self):
        return self._description

    @property
    def entrypoint(self):
        return self._entrypoint


class Annotator(object):

    def __init__(self):
        self._documentation = None
        self._resources = {}
        self._routes = {}
        self._props = {}

    def documentation(self, apidoc_url, title=None, description=None, entrypoint=None):
        self._documentation = Documentation(apidoc_url=apidoc_url, title=title, description=description,
                                            entrypoint=entrypoint)

    def property(self, iri, description=None, routes=None, required=None, readonly=None, writeonly=None,
                 range=None):
        def decorator(meth):
            _prop = Property(name=meth.__name__, iri=iri, description=description, route_keys=routes,
                             required=required, readonly=readonly, writeonly=writeonly, range=range)
            self._props[meth.__name__] = _prop
            return meth

        return decorator

    def resource(self, iri, description=None, routes=None):
        def decorator(cls):
            _resource = Resource(cls=cls, description=description, iri=iri, route_keys=routes)
            _resource.properties = self._props

            self._props = {}
            self._resources[cls.__name__] = _resource
            return cls

        return decorator

    def route(self, name, method, description=None, returns=None, status_codes=None, expects=None):
        self._routes[name] = Route(name=name, method=method, description=description, returns=returns,
                                   expects=expects, status_codes=status_codes)
        def decorator(route_rule):
            return route_rule

        return decorator

    def get_resources(self):
        return self._resources

    def routes(self):
        return self._routes

    def get_properties(self, resource_key):
        if resource_key in self._resources:
            return self._resources[resource_key].properties
        return {}

    def get_routes(self, resource_key):
        if resource_key in self._resources:
            route_keys = self._resources[resource_key].route_keys
            return {route_key: route for (route_key, route) in self._routes.items() if route_key in route_keys}
        return {}

    def get_routes_from_keys(self, route_keys):
        return [route for key, route in self._routes.items() if key in route_keys]

    def get_documentation(self):
        return self._documentation


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# xwot.py - Python tools for the extended Web of Things
# Copyright (C) 2015  Alexander Rüedlinger
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

__author__ = 'Alexander Rüedlinger'

from xwot.model import Context as XWOTContext
from xwot.model import Device as XWOTDevice
from xwot.model import Model
from xwot.model import BaseModel


class Door(XWOTDevice, BaseModel):

    __mutable_props__ = ['name', 'streetAddress', 'roomAddress', 'postalCode', 'addressLocality']
    __expose__ = __mutable_props__ + ['description', 'handle', 'lock']

    def __init__(self, name, street_address, postal_code, address_locality, room_address):
        super(Door, self).__init__()
        self._dic = {
            'name': name,
            'streetAddress': street_address,
            'postalCode': postal_code,
            'addressLocality': address_locality,
            'roomAddress': room_address
        }

        self.add_type('xwot-ext:Door')
        self.add_link('handle')
        self.add_link('lock')

    @property
    def resource_path(self):
        return '/door'

    @property
    def name(self):
        return self._dic['name']

    @property
    def description(self):
        return "Hi there my name is %s. I'm a door and currently present in room %s at the location: %s, %s, %s" % \
               (self.name, self.roomAddress, self.streetAddress, self.addressLocality, self.postalCode)

    @property
    def handle(self):
        return '/door/handle'

    @property
    def lock(self):
        return '/door/lock'

    @property
    def streetAddress(self):
        return self._dic['streetAddress']

    @property
    def postalCode(self):
        return self._dic['postalCode']

    @property
    def addressLocality(self):
        return self._dic['addressLocality']

    @property
    def roomAddress(self):
        return self._dic['roomAddress']


from xwot.i2c.adapter import DoorAdapter


class Lock(XWOTContext, Model):

    __mutable_props__ = ['name', 'state']
    __expose__ = __mutable_props__ + ['description', 'door']

    def __init__(self, name, adapter=DoorAdapter()):
        super(Lock, self).__init__()

        self._dic = {
            'name': name
        }
        self._adapter = adapter
        self.add_type('xwot-ext:Lock')
        self.add_link('door')

    @property
    def resource_path(self):
        return '/door/lock'

    @property
    def door(self):
        return '/door'

    @property
    def description(self):
        return "A door lock which can be locked and unlocked."

    @property
    def state(self):
        return self._adapter.lock_state

    @property
    def name(self):
        return self._dic['name']

    def handle_update(self, dic):
        if dic.get('state') == 'locked':
            self._adapter.lock()

        if dic.get('state') == 'unlocked':
            self._adapter.unlock()

        self._dic['name'] = str(dic.get('name', self._dic['name']))

        return 200


class Handle(XWOTContext, Model):

    __mutable_props__ = ['name', 'state']
    __expose__ = __mutable_props__ + ['description', 'door']

    def __init__(self, name, adapter=DoorAdapter()):
        super(Handle, self).__init__()

        self._dic = {
            'name': name
        }
        self._adapter = adapter
        self.add_type('xwot-ext:Handle')
        self.add_link('door')

    @property
    def resource_path(self):
        return '/door/handle'

    @property
    def door(self):
        return '/door'

    @property
    def description(self):
        return "A door handle which can be closed and opened."

    @property
    def state(self):
        return self._adapter.close_state

    @property
    def name(self):
        return self._dic['name']

    def handle_update(self, dic):
        if dic.get('state') == 'closed':
            self._adapter.close()

        if dic.get('state') == 'opened':
            self._adapter.open()

        self._dic['name'] = str(dic.get('name', self._dic['name']))

        return 200


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# xwot.py - Python tools for the extended Web of Things
# Copyright (C) 2015  Alexander Rüedlinger
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

__author__ = 'Alexander Rüedlinger'

from xwot.i2c.util import Adapter


class DoorAdapter(object):

    CMD_UNLOCK = 0x01
    CMD_LOCK = 0x02
    CMD_OPEN = 0x03
    CMD_CLOSE = 0x04

    CMD_READ_LOCK_STATE = 0x09  # byte value
    CMD_READ_CLOSE_STATE = 0x0A  # byte value

    def __init__(self, bus=1, i2c_addr=0x04):
        self._adapter = Adapter(bus=bus, i2c_addr=i2c_addr)

    def unlock(self):
        return self._adapter.write_byte(self.CMD_UNLOCK)

    def lock(self):
        return self._adapter.write_byte(self.CMD_LOCK)

    def open(self):
        return self._adapter.write_byte(self.CMD_OPEN)

    def close(self):
        return self._adapter.write_byte(self.CMD_CLOSE)

    @property
    def lock_state(self):
        state = self._adapter.read_byte(self.CMD_READ_LOCK_STATE)
        if state == 1:
            return "locked"
        elif state == 0:
            return "unlocked"
        else:
            return None

    @property
    def close_state(self):
        state = self._adapter.read_byte(self.CMD_READ_CLOSE_STATE)
        if state == 1:
            return "closed"
        elif state == 0:
            return "opened"
        elif state == 2:
            return "transition"
        else:
            return None


class LightBulbAdapter(object):

    CMD_SWITCH_LIGHT_ON = 0x01
    CMD_SWITCH_LIGHT_OFF = 0x02

    CMD_READ_LIGHT_BULB_STATE = 0x09  # byte value
    CMD_READ_ILLUMINANCE = 0x0A  # long value (Lux)

    def __init__(self, bus=1, i2c_addr=0x04):
        self._adapter = Adapter(bus=bus, i2c_addr=i2c_addr)

    def switch_on(self):
        return self._adapter.write_byte(self.CMD_SWITCH_LIGHT_ON)

    def switch_off(self):
        return self._adapter.write_byte(self.CMD_SWITCH_LIGHT_OFF)

    @property
    def state(self):
        state = self._adapter.read_byte(self.CMD_READ_LIGHT_BULB_STATE)
        if state == 1:
            return "on"
        elif state == 0:
            return "off"
        else:
            return None

    @property
    def illuminance(self):
        return self._adapter.read_int32(self.CMD_READ_ILLUMINANCE)


class WaterDispenserAdapter(object):

    CMD_OPEN_SOLENOID_VALVE = 0x01
    CMD_CLOSE_SOLENOID_VALVE = 0x02

    CMD_READ_SOIL_MOISTURE = 0x09  # float value (Percentage)
    CMD_READ_SOLENOID_VALVE_STATE = 0x0A  # byte value

    def __init__(self, bus=1, i2c_addr=0x04):
        self._adapter = Adapter(bus=bus, i2c_addr=i2c_addr)

    @property
    def soil_moisture(self):
        return self._adapter.read_float(self.CMD_READ_SOIL_MOISTURE)

    @property
    def solenoid_valve_state(self):
        state = self._adapter.read_byte(self.CMD_READ_SOLENOID_VALVE_STATE)
        if state == 1:
            return "opened"
        elif state == 0:
            return "closed"
        else:
            return None

    def open_solenoid_valve(self):
        self._adapter.write_byte(self.CMD_OPEN_SOLENOID_VALVE)

    def close_solenoid_valve(self):
        self._adapter.write_byte(self.CMD_CLOSE_SOLENOID_VALVE)


class WeatherstationAdapter(object):

    CMD_READ_TEMPERATURE_1 = 0x01  # float value (Celsius)
    CMD_READ_TEMPERATURE_2 = 0x02  # float value (Celsius)
    CMD_READ_PRESSURE = 0x03  # long value (Pascal)
    CMD_READ_HUMIDITY = 0x04  # float value (Percentage)
    CMD_READ_ALTITUDE = 0x05  # long value (Meters)
    CMD_READ_ILLUMINANCE = 0x06  # long value (Lux)
    CMD_READ_COLOR_TEMPERATURE = 0x07  # long value (Kelvin)
    CMD_READ_COLOR_ILLUMINANCE = 0x08  # long value (Lux)

    def __init__(self, bus=1, i2c_addr=0x05):
        self._adapter = Adapter(bus=bus, i2c_addr=i2c_addr)

    @property
    def pressure(self):
        return self._adapter.read_int32(self.CMD_READ_PRESSURE)

    @property
    def altitude(self):
        return self._adapter.read_int32(self.CMD_READ_ALTITUDE)

    @property
    def illuminance(self):
        return self._adapter.read_int32(self.CMD_READ_ILLUMINANCE)

    @property
    def humidity(self):
        return self._adapter.read_float(self.CMD_READ_HUMIDITY)

    @property
    def temperature_1(self):
        return self._adapter.read_float(self.CMD_READ_TEMPERATURE_1)

    @property
    def temperature_2(self):
        return self._adapter.read_float(self.CMD_READ_TEMPERATURE_2)

    @property
    def color_temperature(self):
        return self._adapter.read_int32(self.CMD_READ_COLOR_TEMPERATURE)

    @property
    def color_illuminance(self):
        return self._adapter.read_int32(self.CMD_READ_COLOR_ILLUMINANCE)


class DHTAdapter(object):

    CMD_READ_TEMPERATURE = 0x01  # float value (Celsius)
    CMD_READ_HUMIDITY = 0x02  # float value (Percentage)

    def __init__(self, bus=1, i2c_addr=0x04):
        self._adapter = Adapter(bus=bus, i2c_addr=i2c_addr)

    @property
    def temperature(self):
        return self._adapter.read_float(self.CMD_READ_TEMPERATURE)

    @property
    def humidity(self):
        return self._adapter.read_float(self.CMD_READ_HUMIDITY)


class WindowAdapter(object):

    CMD_UNLOCK = 0x01
    CMD_LOCK = 0x02
    CMD_OPEN = 0x03
    CMD_CLOSE = 0x04

    CMD_READ_LOCK_STATE = 0x09  # byte value
    CMD_READ_CLOSE_STATE = 0x0A  # byte value

    def __init__(self, bus=1, i2c_addr=0x04):
        self._adapter = Adapter(bus=bus, i2c_addr=i2c_addr)

    def unlock(self):
        return self._adapter.write_byte(self.CMD_UNLOCK)

    def lock(self):
        return self._adapter.write_byte(self.CMD_LOCK)

    def open(self):
        return self._adapter.write_byte(self.CMD_OPEN)

    def close(self):
        return self._adapter.write_byte(self.CMD_CLOSE)

    @property
    def lock_state(self):
        state = self._adapter.read_byte(self.CMD_READ_LOCK_STATE)
        if state == 1:
            return "locked"
        elif state == 0:
            return "unlocked"
        else:
            return None

    @property
    def close_state(self):
        state = self._adapter.read_byte(self.CMD_READ_CLOSE_STATE)
        if state == 1:
            return "closed"
        elif state == 0:
            return "opened"
        else:
            return None


class ShutterAdapter(object):

    CMD_UP = 0x01
    CMD_DOWN = 0x02
    CMD_STOP = 0x03

    CMD_READ_STATE = 0x09  # byte value

    def __init__(self, bus=1, i2c_addr=0x05):
        self._adapter = Adapter(bus=bus, i2c_addr=i2c_addr)

    def up(self):
        return self._adapter.write_byte(self.CMD_UP)

    def down(self):
        return self._adapter.write_byte(self.CMD_DOWN)

    def stop(self):
        return self._adapter.write_byte(self.CMD_STOP)

    @property
    def state(self):
        state = self._adapter.read_byte(self.CMD_READ_STATE)
        if state == 1:
            return "up"
        elif state == 0:
            return "down"
        elif state == 2:
            return "transition"
        else:
            return None


class GPSAdapter(object):

    CMD_READ_GPS_STATE = 0x01  # byte value
    CMD_READ_LONGITUDE = 0x02  # float value
    CMD_READ_LATITUDE = 0x03  # float value
    CMD_READ_ELEVATION = 0x04  # float value

    BUFFER_LENGTH = 10

    def __init__(self, bus=1, i2c_addr=0x05):
        self._adapter = Adapter(bus=bus, i2c_addr=i2c_addr)
        self._buffer = {
        'lat': [0] * self.BUFFER_LENGTH,
        'lng': [0] * self.BUFFER_LENGTH,
        'alt': [0] * self.BUFFER_LENGTH
        }
        self._lat_index = 0
        self._lng_index = 0
        self._alt_index = 0
        self._found = False

    def _add_lat(self, value):
        if value is not None and value != -1:
            self._lat_index = (self._lat_index + 1) % self.BUFFER_LENGTH
            self._buffer['lat'][self._lat_index] = value

    def _add_lng(self, value):
        if value is not None and value != -1:
            self._lng_index = (self._lng_index + 1) % self.BUFFER_LENGTH
            self._buffer['lng'][self._lng_index] = value

    def _add_alt(self, value):
        if value is not None:
            self._alt_index = (self._alt_index + 1) % self.BUFFER_LENGTH
            self._buffer['alt'][self._alt_index] = value

    @property
    def state(self):
        return self._found_gps()

    def _found_gps(self):
        _state = self._adapter.read_byte(self.CMD_READ_GPS_STATE)
        if _state:
            return True
        else:
            return False

    @property
    def found(self):
        return self._found_gps()

    @property
    def longitude(self):
        value = self._adapter.read_float(self.CMD_READ_LONGITUDE)
        self._add_lng(value)
        return self._buffer['lng'][self._lng_index]

    @property
    def latitude(self):
        value = self._adapter.read_float(self.CMD_READ_LATITUDE)
        self._add_lat(value)
        return self._buffer['lat'][self._lat_index]

    @property
    def elevation(self):
        value = self._adapter.read_float(self.CMD_READ_ELEVATION)
        self._add_alt(value)
        return self._buffer['alt'][self._alt_index]


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# xwot.py - Python tools for the extended Web of Things
# Copyright (C) 2015  Alexander Rüedlinger
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

__author__ = 'Alexander Rüedlinger'

import os


class OutputPrinter(object):
    """
    OutputPribter is used to generate code.
    """

    def __init__(self):
        # a key in the dictionary represents a file name
        # each key is associated with a list of strings
        # the strings are more or less expressions or statements in some programming language
        self._output = {}

    def indent(self, items):
        return "\n".join([("    %s" % item) for item in items])

    def _add_line(self, item, node_name):
        node_name = tuple(node_name)
        if node_name not in self._output:
            self._output[node_name] = []

        self._output[node_name].append(item)
        self._output[node_name].append("\n")

    def code(self, lines, node_name):
        lines.append("")
        [self._add_line(item, node_name) for item in lines]

    def flatten(self, items):
        return [val for sub_list in items for val in sub_list]

    def flush(self):
        dic = {}
        for node_name, lines in self._output.items():
            out = ''
            for line in lines:
                out += line
            simple_node_name = os.path.join(*node_name)
            dic[simple_node_name] = out

        self._output = {}
        return dic



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# xwot.py - Python tools for the extended Web of Things
# Copyright (C) 2015  Alexander Rüedlinger
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

from __future__ import absolute_import

__author__ = 'Alexander Rüedlinger'
__all__ = ['cors', 'make_response']

SERIALIZERS = {
    'application/json': lambda obj: obj.to_json(),
    'application/xml': lambda obj: obj.to_xml(),
    'text/plain': lambda obj: obj.to_html(),
    'application/ld+json': lambda obj: obj.to_jsonld()
}


def cors(request, origin='*', methods=None, max_age=2520, headers=None):
    if methods is not None:
        methods = ', '.join(sorted(x.upper() for x in methods))
    else:
        methods = 'GET, OPTIONS'

    if headers is not None:
        headers = ', '.join(headers)
    else:
        headers = 'origin,x-prototype-version,x-requested-with,content-type,accept'

    request.setHeader('Access-Control-Allow-Origin', origin)
    request.setHeader('Access-Control-Allow-Methods', methods)
    request.setHeader('Access-Control-Allow-Headers', headers)
    request.setHeader('Access-Control-Max-Age', max_age)


def make_response(obj, request, default='application/ld+json', status=200):
    content_type = request.getHeader('Accept')

    if content_type in SERIALIZERS:
        fun_serializer = SERIALIZERS[content_type]
        doc = fun_serializer(obj)
        request.setHeader('Content-Type', content_type)
        request.setResponseCode(status)
        return doc
    else:
        fun_serializer = SERIALIZERS[default]
        doc = fun_serializer(obj)
        request.setHeader('Content-Type', default)
        request.setResponseCode(status)
        return doc


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# xwot.py - Python tools for the extended Web of Things
# Copyright (C) 2015  Alexander Rüedlinger
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

__author__ = 'Alexander Rüedlinger'

from xwot.model import Context as XWOTContext
from xwot.model import Sensor as XWOTSensor
from xwot.model import Device as XWOTDevice
from xwot.model import Model
from xwot.model import BaseModel


class LightBulb(XWOTDevice, BaseModel):

    __mutable_props__ = ['name', 'streetAddress', 'roomAddress', 'postalCode', 'addressLocality']
    __expose__ = __mutable_props__ + ['description', 'switch', 'sensor']

    def __init__(self, name, street_address, postal_code, address_locality, room_address):
        super(LightBulb, self).__init__()
        self._dic = {
            'name': name,
            'streetAddress': street_address,
            'postalCode': postal_code,
            'addressLocality': address_locality,
            'roomAddress': room_address
        }

        self.add_type('xwot-ext:LightBulb')
        self.add_link('switch')
        self.add_link('sensor')

    @property
    def resource_path(self):
        return '/lightbulb'

    @property
    def name(self):
        return self._dic['name']

    @property
    def description(self):
        return "Hi there my name is %s. I'm a light bulb and currently present in room %s at the location: %s, %s, %s" % \
               (self.name, self.roomAddress, self.streetAddress, self.addressLocality, self.postalCode)

    @property
    def switch(self):
        return '/lightbulb/switch'

    @property
    def sensor(self):
        return '/lightbulb/sensor'

    @property
    def streetAddress(self):
        return self._dic['streetAddress']

    @property
    def postalCode(self):
        return self._dic['postalCode']

    @property
    def addressLocality(self):
        return self._dic['addressLocality']

    @property
    def roomAddress(self):
        return self._dic['roomAddress']


from xwot.i2c.adapter import LightBulbAdapter


class Switch(XWOTContext, Model):

    __mutable_props__ = ['name', 'state']
    __expose__ = __mutable_props__ + ['description', 'lightbulb']

    def __init__(self, name, adapter=LightBulbAdapter()):
        super(Switch, self).__init__()

        self._dic = {
            'name': name
        }
        self._adapter = adapter
        self.add_type('xwot-ext:Switch')
        self.add_link('lightbulb')

    @property
    def resource_path(self):
        return '/lightbulb/switch'

    @property
    def description(self):
        return "A light switch to turn off or on."

    @property
    def lightbulb(self):
        return '/lightbulb'

    @property
    def state(self):
        return self._adapter.state

    @property
    def name(self):
        return self._dic['name']

    def handle_update(self, dic):
        if dic.get('state') == 'off':
            self._adapter.switch_off()

        if dic.get('state') == 'on':
            self._adapter.switch_on()

        self._dic['name'] = str(dic.get('name', self._dic['name']))

        return 200


class Sensor(XWOTSensor, Model):

    __expose__ = ['name', 'unit', 'measures', 'description', 'measurement', 'symbol', 'lightbulb']

    def __init__(self, adapter=LightBulbAdapter()):
        super(Sensor, self).__init__()
        self._adapter = adapter
        self.add_type('xwot-ext:IlluminanceSensor')
        self.add_link('lightbulb')

    @property
    def resource_path(self):
        return '/lightbulb/sensor'

    @property
    def name(self):
        return 'Illuminance sensor'

    @property
    def lightbulb(self):
        return '/lightbulb'

    @property
    def unit(self):
        return 'Lux'

    @property
    def description(self):
        return 'A sensor that measures the illuminance of this light bulb.'

    @property
    def measures(self):
        return 'Illuminance'

    @property
    def measurement(self):
        return self._adapter.illuminance

    @property
    def symbol(self):
        return 'lx'

    def handle_update(self, dic):
        pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# xwot.py - Python tools for the extended Web of Things
# Copyright (C) 2015  Alexander Rüedlinger
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

__author__ = 'Alexander Rüedlinger'


class Iri(object):

    __iri__ = True

    def __init__(self, iri):
        self._iri = iri

    def __str__(self):
        return self._iri


class NullIri(Iri):

    def __init__(self):
        super(NullIri, self).__init__(None)



class Owl(object):
    NOTHING = Iri("http://www.w3.org/2002/07/owl#Nothing")

    @classmethod
    def Nothing(cls):
        return cls.NOTHING


class Xsd(object):

    BOOLEAN = Iri("http://www.w3.org/2001/XMLSchema#boolean")
    STRING = Iri("http://www.w3.org/2001/XMLSchema#string")
    DATE_TIME = Iri("http://www.w3.org/2001/XMLSchema#dateTime")
    INTEGER = Iri("http://www.w3.org/2001/XMLSchema#integer")
    FLOAT = Iri("http://www.w3.org/2001/XMLSchema#float")
    DOUBLE = Iri("http://www.w3.org/2001/XMLSchema#double")
    LONG = Iri("http://www.w3.org/2001/XMLSchema#long")

    @classmethod
    def string(cls):
        return cls.STRING

    @classmethod
    def boolean(cls):
        return cls.BOOLEAN

    @classmethod
    def datetime(cls):
        return cls.DATE_TIME

    @classmethod
    def integer(cls):
        return cls.INTEGER

    @classmethod
    def double(cls):
        return cls.DOUBLE

    @classmethod
    def float(cls):
        return cls.FLOAT

    @classmethod
    def long(cls):
        return cls.LONG


class Hydra(object):

    RDF_PROPERTY = Iri('http://www.w3.org/1999/02/22-rdf-syntax-ns#Property')
    HYDRA = Iri("http://www.w3.org/ns/hydra/core#")
    HYDRA_COLLECTION = Iri("http://www.w3.org/ns/hydra/core#Collection")
    HYDRA_LINK = Iri("http://www.w3.org/ns/hydra/core#Link")
    HYDRA_TITLE = Iri('http://www.w3.org/ns/hydra/core#title')
    HYDRA_DESCRIPTION = Iri('http://www.w3.org/ns/hydra/core#description')
    HYDRA_CLASS = Iri('http://www.w3.org/ns/hydra/core#Class')
    HYDRA_OPERATION = Iri('http://www.w3.org/ns/hydra/core#Operation')
    HYDRA_MEMBER = Iri('http://www.w3.org/ns/hydra/core#member')

    @classmethod
    def core(cls):
        return cls.HYDRA

    @classmethod
    def Collection(cls):
        return cls.HYDRA_COLLECTION

    @classmethod
    def Link(cls):
        return cls.HYDRA_LINK

    @classmethod
    def Class(cls):
        return cls.HYDRA_CLASS

    @classmethod
    def Property(cls):
        return cls.RDF_PROPERTY

    @classmethod
    def member(cls):
        return cls.HYDRA_MEMBER


class SchemaOrg(object):

    DESCRIPTION = Iri("http://schema.org/description")
    NAME = Iri("http://schema.org/name")
    EMAIL = Iri("http://schema.org/email")
    URL = Iri("http://schema.org/url")
    PERSON = Iri("http://schema.org/Person")
    TEXT = Iri("http://schema.org/Text")
    ACCESS_CODE = Iri("http://schema.org/accessCode")


    @classmethod
    def Text(cls):
        return cls.TEXT

    @classmethod
    def accessCode(cls):
        return cls.ACCESS_CODE

    @classmethod
    def description(cls):
        return cls.DESCRIPTION

    @classmethod
    def name(cls):
        return cls.NAME

    @classmethod
    def email(cls):
        return cls.EMAIL

    @classmethod
    def url(cls):
        return cls.URL

    @classmethod
    def Person(cls):
        return cls.PERSON


class Xwot(object):

    ENTITY = Iri("http://xwot.lexruee.ch/vocab/core#Entity")
    CONTEXT = Iri("http://xwot.lexruee.ch/vocab/core#Context")
    SENSOR = Iri("http://xwot.lexruee.ch/vocab/core#Sensor")
    SERVICE = Iri("http://xwot.lexruee.ch/vocab/core#Service")
    ACTUATOR = Iri("http://xwot.lexruee.ch/vocab/core#Actuator")
    TAG = Iri("http://xwot.lexruee.ch/vocab/core#Tag")
    PUBLISHER = Iri("http://xwot.lexruee.ch/vocab/core#Publisher")
    DEVICE = Iri("http://xwot.lexruee.ch/vocab/core#Device")
    RESOURCE = Iri("http://xwot.lexruee.ch/vocab/core#Resource")
    DESCRIPTION = Iri("http://xwot.lexruee.ch/vocab/core#Description")

    @classmethod
    def Description(cls):
        return cls.DESCRIPTION

    @classmethod
    def Entity(cls):
        return cls.ENTITY

    @classmethod
    def Context(cls):
        return cls.CONTEXT

    @classmethod
    def Sensor(cls):
        return cls.SENSOR

    @classmethod
    def Actuator(cls):
        return cls.ACTUATOR

    @classmethod
    def Service(cls):
        return cls.SERVICE

    @classmethod
    def Publisher(cls):
        return cls.PUBLISHER

    @classmethod
    def Device(cls):
        return cls.DEVICE

    @classmethod
    def Resource(cls):
        return cls.RESOURCE

    @classmethod
    def Tag(cls):
        return cls.TAG


#!/usr/bin/env python
# -*- coding: utf-8 -*-

# #############################################################################################################
# Takes a xwot specification adds the virtual parts #
# ---------------------------------------------------------------------------------------------------------- #
# #
# Author: Andreas Ruppen                                                                                     #
# License: GPL                                                                                               #
# This program is free software; you can redistribute it and/or modify                                       #
# it under the terms of the GNU General Public License as published by                                     #
#   the Free Software Foundation; either version 2 of the License, or                                        #
#   (at your option) any later version.                                                                      #
#                                                                                                            #
#   This program is distributed in the hope that it will be useful,                                          #
#   but WITHOUT ANY WARRANTY; without even the implied warranty of                                           #
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                                            #
#   GNU General Public License for more details.                                                             #
#                                                                                                            #
#   You should have received a copy of the GNU General Public License                                        #
#   along with this program; if not, write to the                                                            #
#   Free Software Foundation, Inc.,                                                                          #
#   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.                                                #
##############################################################################################################
import sys
import logging
import logging.config
import os
import xml.dom.minidom
import argparse
import codecs
from colorama import Fore, Back, Style
from os.path import dirname, join, expanduser

if float(sys.version[:3]) < 3.0:
    import ConfigParser
else:
    import configparser as ConfigParser


class Physical2VirtualEntities:
    """
        Created on 23 sep. 2013
        @author: ruppena
    """

    def __init__(self):
        """Do some initialization stuff"""
        INSTALL_DIR = dirname(__file__)
        CONFIG_DIR = '/etc/Model2WADL/'
        logging.basicConfig(level=logging.ERROR)
        logging.config.fileConfig([join(CONFIG_DIR, 'logging.conf'), expanduser('~/.logging.conf'), 'logging.conf'])
        self.__log = logging.getLogger('thesis')

        self.__log.debug("Reading general configuration from Model2WADL.cfg")
        self.__m2wConfig = ConfigParser.SafeConfigParser()
        self.__m2wConfig.read(
            [join(CONFIG_DIR, 'Physical2Virtual.cfg'), expanduser('~/.Physical2Virtual.cfg'), 'Physical2Virtual.cfg'])

        self.__baseURI = self.__m2wConfig.get("Config", "baseURI")
        self.__basePackage = self.__m2wConfig.get("Config", "basePackage")
        self.__schemaFile = self.__m2wConfig.get("Config", "schemaFile")
        self.__model = None
        self.__input = None
        self.__output = None

    def __setupxWoT(self):
        self.__xwot = xml.dom.minidom.Document()

        rootElement = self.__xwot.createElementNS('http://diuf.unifr.ch/softeng', 'xwot:Entity')

        rootElement.setAttribute("xmlns:xmi", "http://www.omg.org/XMI")
        rootElement.setAttribute("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance")
        rootElement.setAttribute("xmlns:xwot", "http://diuf.unifr.ch/softeng")
        rootElement.setAttribute("xmi:version", "2.0")
        self.__xwot.appendChild(rootElement)

    def __createVirtualEntities(self):
        """For each physical device a virtual counterpart is created. Since some combinations of Sensor/Actuator result in a Context resource, the user is asked for some input"""
        root = self.__xwot.documentElement

        ve = self.__xwot.createElement('VirtualEntity')
        physicalEntity = self.__model.getElementsByTagName('PhysicalEntity')[0]
        newPhysEnt = physicalEntity.cloneNode(False)
        root.appendChild(newPhysEnt)
        elType = physicalEntity.getAttribute('xsi:type')
        name = physicalEntity.getAttribute('name')
        ve.setAttribute('name', name + 'Resource')
        uri = raw_input('Specify URI for ' + Fore.RED + elType + ' ' + Fore.GREEN + name + Fore.RESET + ': ')
        ve.setAttribute('uri', uri)
        if elType == 'xwot:Device':
            ve.setAttribute('xsi:type', 'xwot:Resource')
            self.__log.debug(physicalEntity.attributes)
            if  physicalEntity.hasAttribute('composed'):
                ve.setAttribute('composed',  physicalEntity.getAttribute('composed'))
            root.appendChild(ve)
            physicalEntities = list(
                self.__filterChildrenByTagName(self.__model.getElementsByTagName('PhysicalEntity')[0], 'Component'))
            physicalEntities = self.__searchForContextResources(physicalEntities, newPhysEnt, ve)
            for entity in physicalEntities:
                self.__log.debug('Working on Node: ' + entity.getAttribute('name'))
                self.__addResources(newPhysEnt, entity, ve)
        elif elType == 'xwot:Sensor':
            ve.setAttribute('xsi:type', 'xwot:SensorResource')
            root.appendChild(ve)
            answer = raw_input('Has this Sensor a publisher? ' + Fore.RED + '[y/n] ' + Fore.RESET + '?')
            if answer in ('y', 'Y', 'Yes', 'yes', 'YES'):
                self.__createPublisherResource(ve)
        elif elType == 'xwot:Actuator':
            ve.setAttribute('xsi:type', 'xwot:ActuatorResource')
            root.appendChild(ve)

    def __addResources(self, targetPhysicalEntity, sourceNode, targetNode):
        vent = self.__xwot.createElement('Resource')
        ename = sourceNode.getAttribute('name')
        etype = sourceNode.getAttribute('xsi:type')
        vent.setAttribute('name', ename + 'Resource')
        uri = raw_input('Specify URI for ' + Fore.RED + etype + ' ' + Fore.GREEN + ename + Fore.RESET + ': ')
        vent.setAttribute('uri', uri)
        newTargetPhysicalEntity = sourceNode.cloneNode(False)
        targetPhysicalEntity.appendChild(newTargetPhysicalEntity)
        if etype == 'xwot:Device':
            vent.setAttribute('xsi:type', 'xwot:Resource')
            if  sourceNode.hasAttribute('composed'):
                vent.setAttribute('composed',  sourceNode.getAttribute('composed'))
            targetNode.appendChild(vent)
            subPhysicalEntities = list(self.__filterChildrenByTagName(sourceNode, 'Component'))
            subPhysicalEntities = self.__searchForContextResources(subPhysicalEntities, newTargetPhysicalEntity, vent)
            for subEntity in subPhysicalEntities:
                self.__log.debug('Working on Node: ' + subEntity.getAttribute('name'))
                self.__addResources(newTargetPhysicalEntity, subEntity, vent)
        elif etype == 'xwot:Sensor':
            vent.setAttribute('xsi:type', 'xwot:SensorResource')
            targetNode.appendChild(vent)
            answer = raw_input('Has this Sensor a publisher? ' + Fore.RED + '[y/n] ' + Fore.RESET + '?')
            if answer in ('y', 'Y', 'Yes', 'yes', 'YES'):
                self.__createPublisherResource(vent)
        elif etype == 'xwot:Actuator':
            vent.setAttribute('xsi:type', 'xwot:ActuatorResource')
            targetNode.appendChild(vent)

    @staticmethod
    def __filterChildrenByTagName(node, tagName):
        for child in node.childNodes:
            if child.nodeType == child.ELEMENT_NODE and child.tagName == tagName:
                yield child

    def __searchForContextResources(self, inputNodeList, targetPhysicalEntity, parentDestinationNode):
        doAskAgain = (True if len(inputNodeList) > 1 else False)
        while doAskAgain and len(inputNodeList) > 1:
            print('I have found the following Nodes:')
            self.__printChildren(inputNodeList)
            answer = raw_input('Is there a ContextResource? ' + Fore.RED + '[y/n]' + Fore.RESET + '?')
            if answer not in ('y', 'Y', 'Yes', 'yes', 'YES'):
                doAskAgain = False
                continue
            if len(inputNodeList) > 2:
                numbers = raw_input('Input comma separated numbers of ContextResource: ')
                firstElement = inputNodeList[(int(numbers.rsplit(',')[0]))]
                secondElement = inputNodeList[(int(numbers.rsplit(',')[1]))]
            else:
                firstElement = inputNodeList[0]
                secondElement = inputNodeList[1]
            inputNodeList.remove(firstElement)
            inputNodeList.remove(secondElement)

            print('Combining Node ' + firstElement.getAttribute('name') + ' and Node ' + secondElement.getAttribute(
                'name') + ' into ContextResource')
            newTargetPhysicalEntity = firstElement.cloneNode(False)
            targetPhysicalEntity.appendChild(newTargetPhysicalEntity)
            newTargetPhysicalEntity = secondElement.cloneNode(False)
            targetPhysicalEntity.appendChild(newTargetPhysicalEntity)
            self.__createContextResource(firstElement, secondElement, parentDestinationNode)

        return inputNodeList

    def __createContextResource(self, firstElement, secondElement, parentDestinationNode):
        vent = self.__xwot.createElement('Resource')
        ename = firstElement.getAttribute('name') + secondElement.getAttribute('name')

        vent.setAttribute('name', ename + 'ContextResource')
        uri = raw_input('Specify URI for ContextResource ' + Fore.GREEN + ename + Fore.RESET + ': ')
        vent.setAttribute('uri', uri)
        vent.setAttribute('xsi:type', 'xwot:ContextResource')
        parentDestinationNode.appendChild(vent)
        answer = raw_input('Has this ContextResource a publisher? ' + Fore.RED + '[y/n] ' + Fore.RESET + '?')
        if answer in ('y', 'Y', 'Yes', 'yes', 'YES'):
            self.__createPublisherResource(vent)

    def __createPublisherResource(self, parentDestinationNode):
        ename = parentDestinationNode.getAttribute('name')
        publisher = self.__xwot.createElement('Resource')
        publisher.setAttribute('xsi:type', 'xwot:PublisherResource')
        publisher.setAttribute('name', ename + 'PublisherResource')
        publisher.setAttribute('uri', 'pub')
        parentDestinationNode.appendChild(publisher)

    @staticmethod
    def __printChildren(nodes):
        i = 0
        for node in nodes:
            print(str(i) + ': ' + node.toxml())
            i += 1

    def simplify(self):
        virtualEntity = self.__xwot.getElementsByTagName('VirtualEntity')[0]
        resources = self.__filterChildrenByTagName(virtualEntity, 'Resource')
        for resource in resources:
            self.__doSimplify(resource)

    def __doSimplify(self, resource):
        self.__log.debug('Working on Node: ' + resource.getAttribute('name'))
        self.__log.debug('Has ' + str(len(resource.childNodes)) + ' children')
        if len(resource.childNodes) == 1 and resource.childNodes[0].getAttribute('uri') != 'pub':
            child = resource.childNodes[0].cloneNode(True)
            parent = resource.parentNode
            self.__log.debug(
                'Will replace node ' + resource.getAttribute('name') + ' with node ' + child.getAttribute('name'))
            parent.replaceChild(child, resource)
            self.__doSimplify(child)
        else:
            resources = self.__filterChildrenByTagName(resource, 'Resource')
            for r in resources:
                self.__doSimplify(r)

    def main(self):
        """The main function"""
        self.__log.debug("current working directory is: " + os.getcwd())
        self.__log.debug("input File is: " + self.__input)
        self.__log.debug("output File is: " + self.__output)
        self.__setupxWoT()
        self.__model = xml.dom.minidom.parse(self.__input)
        self.__createVirtualEntities()

        self.simplify()
        f = codecs.open(self.__output, 'w', 'utf-8')
        self.__xwot.writexml(f, indent="", addindent="\t", newl="\n", encoding="UTF-8")

    def getArguments(self, argv):
        parser = argparse.ArgumentParser()
        parser.add_argument("-i", "--input", help="input xwot file containing the Model to be translated",
                            required=True)
        parser.add_argument("-o", "--output",
                            help="output xwot file containing the Model enhanced with the virtual entities",
                            required=True)
        args = parser.parse_args(argv)
        self.__input = args.input
        self.__output = args.output
        try:
            self.__log.info("Start processing")
            self.main()
            self.__log.info("Successfully created the necessary service(s)")
        except Exception as err:
            self.__log.error("Something went really wrong")
            self.__log.debug(err)


if __name__ == "__main__":
    prog = Physical2VirtualEntities()
    prog.getArguments(sys.argv[1:])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# xwot.py - Python tools for the extended Web of Things
# Copyright (C) 2015  Alexander Rüedlinger
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

__author__ = 'Alexander Rüedlinger'

from xwot.util.serializer import pretty_json
import logging

logger = logging.getLogger('hydra vocab builder')


class HydraBuilder(object):
    CONTEXT = {
        "vocab": "missing",
        "hydra": "http://www.w3.org/ns/hydra/core#",
        "ApiDocumentation": "hydra:ApiDocumentation",
        "property": {
            "@id": "hydra:property",
            "@type": "@id"
        },
        "readonly": "hydra:readonly",
        "writeonly": "hydra:writeonly",
        "supportedClass": "hydra:supportedClass",
        "supportedProperty": "hydra:supportedProperty",
        "supportedOperation": "hydra:supportedOperation",
        "method": "hydra:method",
        "expects": {
            "@id": "hydra:expects",
            "@type": "@id"
        },
        "returns": {
            "@id": "hydra:returns",
            "@type": "@id"
        },
        "statusCodes": "hydra:statusCodes",
        "code": "hydra:statusCode",
        "rdf": "http://www.w3.org/1999/02/22-rdf-syntax-ns#",
        "rdfs": "http://www.w3.org/2000/01/rdf-schema#",
        "label": "rdfs:label",
        "description": "rdfs:comment",
        "domain": {
            "@id": "rdfs:domain",
            "@type": "@id"
        },
        "range": {
            "@id": "rdfs:range",
            "@type": "@id"
        },
        "subClassOf": {
            "@id": "rdfs:subClassOf",
            "@type": "@id"
        }
    }

    def __init__(self, annotator):
        self._annotator = annotator
        self._contexts = {}
        self._documentation = annotator.get_documentation()

    @property
    def documentation(self):
        return self._annotator.get_documentation()

    def build(self):
        return self._build_api_doc()

    def _build_api_doc(self):
        resources = self._visit_resources()
        new_context = self.CONTEXT.copy()
        new_context['vocab'] = self._documentation.apidoc_url + '#'

        if self._documentation is None:
            raise Exception

        api_doc = {
            '@context': new_context,
            '@id': self._documentation.apidoc_url,
            '@type': 'ApiDocumentation',
            'supportedClass': resources
        }

        jsonld_doc = pretty_json(api_doc)
        with open('test.json', 'w+') as f:
            f.write(jsonld_doc)

        return jsonld_doc

    def _visit_resources(self):
        resources = []

        for resource_key, resource in self._annotator.get_resources().items():
            self._contexts[resource_key] = {
                'vocab': self._documentation.apidoc_url
            }
            routes = self._annotator.get_routes(resource_key)
            props = self._annotator.get_properties(resource_key)
            supported_operations = self._visit_routes(resource, routes)
            supported_properties = self._visit_properties(resource, props)

            resource_dic = {
                '@id': resource.iri.__str__(),
                '@type': 'hydra:Class',
                'label': resource.name,
                'description': resource.description,
                'supportedOperation': supported_operations,
                'supportedProperty': supported_properties
            }

            resources.append(resource_dic)

        return resources

    def _visit_routes(self, resource, routes):
        route_dics = []
        for route_key, route in routes.items():
            operation = self._create_route_dic(route_key, route)
            route_dics.append(operation)

        return route_dics

    def _create_route_dic(self, route_key, route):
        route_dic = {
            '@id': "_:%s" % route_key,
            '@type': 'hydra:Operation',
            'method': route.method,
            'label': route.description,
            'description': route.description,
            'expects': route.expects.__str__(),
            'returns': route.returns.__str__(),
            'statusCodes': route.status_codes
        }
        return route_dic

    def _visit_properties(self, resource, props):
        properties = []
        for prop_key, prop in props.items():
            prop_def = self._visit_iri(resource, prop)
            prop_dic = {
                'property': prop_def,
                'hydra:title': prop_key,
                'hydra:description': prop.description,
                'required': prop.required,
                'readonly': prop.readonly,
                'writeonly': prop.writeonly
            }
            properties.append(prop_dic)

        return properties

    def _visit_iri(self, resource, prop):
        _iri = prop.iri.__str__()
        prop_def = _iri

        if len(prop.route_keys) > 0:
            _routes = self._annotator.get_routes_from_keys(prop.route_keys)
            operations = [self._create_route_dic(route.name, route) for route in _routes]
            _first_route = _routes[0]
            _range = _first_route.returns.__str__()
            _domain = resource.iri.__str__()

            prop_def = {
                '@id': _iri,
                '@type': 'hydra:Link',
                'label': prop.name,
                'description': prop.description,
                'domain': _domain,
                'range': _range,
                'supportedOperation': operations
            }

        return prop_def




import models
from xwot.util.annotator import Annotator
from xwot.util.serializer import Serializer
from xwot.util import local_ip
annotator = Annotator()


serializer = Serializer(annotator)

entrypoint = models.EntryPoint()

url = "http://%s:%s" % (local_ip(), 3000)
output = serializer.serialize(entrypoint, content_type='text/html')

print(output)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# xwot.py - Python tools for the extended Web of Things
# Copyright (C) 2015  Alexander Rüedlinger
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>


__author__ = 'Alexander Rüedlinger'

from helper import OutputPrinter
from xwot.compiler.frontend import visitor




class KleinBackendBuilder(visitor.BaseVisitor):
    def __init__(self):
        visitor.BaseVisitor.__init__(self)
        self._out = OutputPrinter()

    def _create_symbol_name(self, path):
        return path.replace("/", "_").replace("-", "_").replace(".", "_").replace('<', '').replace('>', '')

    def before(self):
        self._out.code([
                           '#',
                           '# Generated by xwot compiler.',
                           '#',
                           '# Klein xwot application.',
                           '#',
                           '',
                           'import os',
                           'from yadp.device import Description',
                           '',
                           'from xwot.util import local_ip',
                           'from xwot.util import create_description',
                           'from xwot.util import dir_path',
                           'from xwot.util import parent_dir_path',
                           '',
                           '# base config',
                           'ip = local_ip()',
                           'port = 5000',
                           'http_addr = "http://%s:%s/" % (ip, port)',
                           'module_dir_path = dir_path(__file__)',
                           'app_dir_path = parent_dir_path(__file__)',
                           'xwot_file = os.path.join(app_dir_path, "device.xwot")',
                           '',
                           'jsonld_description_str = create_description(xwot_file=xwot_file, base=http_addr)',
                           'yadp_description = Description(content_type="application/ld+json", content=jsonld_description_str)',
                           '',
                           '',
                           'from klein import Klein',
                           'app = Klein()',
                           ], ["xwot_app", "__init__"])



    def after(self):
        self._out.code([
                           "import %s" % node.name() for node in self._nodes
                       ], ["xwot_app", "__init__"])

        self._out.code([
                           '#',
                           '# Generated by xwot compiler.',
                           '#',
                           '# Klein xwot application.',
                           '#',
                           '',
                           'import xwot_app',
                           'from xwot_app import app',
                           'from xwot.util.klein import make_response',
                           'from xwot.util import deserialize',
                           'from xwot.util.klein import cors',
                           'from twisted.internet import task, reactor',
                           '',
                           'import yadp',
                           'yadp.debug()',
                           '',
                           'from yadp import service',
                           'from yadp.device import Device',
                           '',
                           "device = Device(urn='urn:xwot:Device', location=xwot_app.http_addr, descriptions=[xwot_app.yadp_description])",
                           '',
                           'service = service()',
                           'service.register(device=device, passive=True)',
                           '',
                           "",
                           "if __name__ == '__main__':",
                           self._out.indent([
                               "app.run(host='0.0.0.0', port=xwot_app.port)"
                           ])
                       ], ['runserver'])

    def before_resource(self, node):
        self._out.code([
                           '#',
                           '# Generated by xwot compiler.',
                           '#',
                           '# Klein xwot application.',
                           '#',
                           "# Type:       %s" % node.type(),
                           "# Resource:   %s" % node.name(),
                           "# Path:       %s" % node.fullpath(),
                           '#',
                           '',
                           'from xwot_app import app',
                            'from xwot_app import app',
                            'from xwot.util.klein import make_response',
                            'from xwot.util import deserialize',
                            'from xwot.util.klein import cors',
                            'from twisted.internet import task, reactor',
                           ''
                       ], ["xwot_app", node.name()])

    def handle_entity(self, node):
        self._out.code([
                           '#',
                           '# Generated by compiler compiler.',
                           '#',
                           '# Klein xwot application.',
                           '#',
                           "# Type:       %s" % node.type(),
                           "# Resource:   %s" % node.name(),
                           "# Path:       %s" % node.fullpath(),
                           '#',
                           '',
                           'import xwot_app',
                           'from xwot_app import app',
                           'from twisted.web.static import File',
                           '',
                           '',
                           "@app.route('/')",
                           'def home(request):',
                           self._out.indent([
                               "request.setHeader('Content-Type', 'application/ld+json')",
                               'return xwot_app.jsonld_description_str'
                           ]),
                           '',
                           '',
                           "@app.route('/static/', branch=True)",
                           "def static_files(request):",
                           self._out.indent([
                            'return File("./static")'
                            ]),
                           '',
                           '',

                       ], ["xwot_app", node.name()])

    def handle_resource(self, node):
        for method in ['GET', 'PUT', 'OPTIONS']:
            self._out.code([
                               '#',
                               "# %s '%s'" % (method.upper(), node.fullpath()),
                               '#',
                               "@app.route('%s', methods=['%s'])" % (node.fullpath(), method),
                               "def handle%s_%s(request):" % (self._create_symbol_name(node.fullpath()), method),
                               self._out.indent([
                                   "cors(request, methods=['GET', 'PUT', 'OPTIONS'])",
                                   "request.setHeader('Allow', 'GET, PUT, OPTIONS')",
                                   'return "Name: %s , Hello at: %s"' % (node.name(), node.fullpath())
                               ])
                           ], ["xwot_app", node.name()])

    def handle_device_resource(self, node):
        for method in ['GET', 'PUT', 'OPTIONS']:
            self._out.code([
                               '#',
                               "# %s '%s'" % (method.upper(), node.fullpath()),
                               '#',
                               "@app.route('%s', methods=['%s'])" % (node.fullpath(), method),
                               "def handle%s_%s(request):" % (self._create_symbol_name(node.fullpath()), method),
                               self._out.indent([
                                   "cors(request, methods=['GET', 'PUT', 'OPTIONS'])",
                                   "request.setHeader('Allow', 'GET, PUT, OPTIONS')",
                                   'return "Name: %s , Hello at: %s"' % (node.name(), node.fullpath())
                               ])
                           ], ["xwot_app", node.name()])

    def handle_sensor_resource(self, node):
        for method in ['GET', 'OPTIONS']:
            self._out.code([
                               '#',
                               "# %s '%s'" % (method.upper(), node.fullpath()),
                               '#',
                               "@app.route('%s', methods=['%s'])" % (node.fullpath(), method),
                               "def handle%s_%s(request):" % (self._create_symbol_name(node.fullpath()), method),
                               self._out.indent([
                                   "cors(request, methods=['GET', 'PUT', 'OPTIONS'])",
                                   "request.setHeader('Allow', 'GET, PUT, OPTIONS')",
                                   'return "Name: %s , Hello at: %s"' % (node.name(), node.fullpath())
                               ])
                           ], ["xwot_app", node.name()])

    def handle_tag_resource(self, node):
        for method in ['GET', 'OPTIONS']:
            self._out.code([
                               '#',
                               "# %s '%s'" % (method.upper(), node.fullpath()),
                               '#',
                               "@app.route('%s', methods=['%s'])" % (node.fullpath(), method),
                               "def handle%s_%s(request):" % (self._create_symbol_name(node.fullpath()), method),
                               self._out.indent([
                                   "cors(request, methods=['GET', 'PUT', 'OPTIONS'])",
                                   "request.setHeader('Allow', 'GET, PUT, OPTIONS')",
                                   'return "Name: %s , Hello at: %s"' % (node.name(), node.fullpath())
                               ])
                           ], ["xwot_app", node.name()])

    def handle_context_resource(self, node):
        for method in ['GET', 'PUT', 'OPTIONS']:
            self._out.code([
                               '#',
                               "# %s '%s'" % (method.upper(), node.fullpath()),
                               '#',
                               "@app.route('%s', methods=['%s'])" % (node.fullpath(), method),
                               "def handle%s_%s(request):" % (self._create_symbol_name(node.fullpath()), method),
                               self._out.indent([
                                   "cors(request, methods=['GET', 'PUT', 'OPTIONS'])",
                                   "request.setHeader('Allow', 'GET, PUT, OPTIONS')",
                                   'return "Name: %s , Hello at: %s"' % (node.name(), node.fullpath())
                               ])
                           ], ["xwot_app", node.name()])

    def handle_service_resource(self, node):
        for method in ['GET', 'POST', 'PUT', 'DELETE', 'OPTIONS']:
            self._out.code([
                               '#',
                               "# %s '%s'" % (method.upper(), node.fullpath()),
                               '#',
                               "@app.route('%s', methods=['%s'])" % (node.fullpath(), method),
                               "def handle%s_%s(request):" % (self._create_symbol_name(node.fullpath()), method),
                               self._out.indent([
                                   "cors(request, methods=['GET', 'PUT', 'OPTIONS'])",
                                   "request.setHeader('Allow', 'GET, PUT, OPTIONS')",
                                   'return "Name: %s , Hello at: %s"' % (node.name(), node.fullpath())
                               ])
                           ], ["xwot_app", node.name()])

    def handle_actuator_resource(self, node):
        for method in ['PUT', 'OPTIONS']:
            self._out.code([
                               '#',
                               "# %s '%s'" % (method.upper(), node.fullpath()),
                               '#',
                               "@app.route('%s', methods=['%s'])" % (node.fullpath(), method),
                               "def handle%s_%s(request):" % (self._create_symbol_name(node.fullpath()), method),
                               self._out.indent([
                                   "cors(request, methods=['GET', 'PUT', 'OPTIONS'])",
                                   "request.setHeader('Allow', 'GET, PUT, OPTIONS')",
                                   'return "Name: %s , Hello at: %s"' % (node.name(), node.fullpath())
                               ])
                           ], ["xwot_app", node.name()])

    def handle_publisher_resource(self, node):
        self._out.code([
            'import json',
            'import logging',
            'from templates.tpl_client import ClientElement',
            'from templates.tpl_clients import ClientsElement',
            'from twisted.web.template import flattenString',
            'from autobahn.twisted.resource import WebSocketResource, HTTPChannelHixie76Aware',
            'from xwot.util.WebSocketSupport import xWoTStreamerProtocol',
            'from xwot.util.WebSocketSupport import xWoTBroadcastFactory',
            'from  xwot.util.SubscriberDB import SubscriberDB',
            'from xwot.util.SampleHardware import SampleHardware'
        ], ["xwot_app", node.name()])

        self._out.code([
            '#',
           "# %s '%s'" % ('GET', node.fullpath()),
           '#',
           "@app.route('%s', methods=['%s'])" % (node.fullpath(), 'GET'),
           "def handle%s_%s(request):" % (self._create_symbol_name(node.fullpath()), 'GET'),
            self._out.indent([
                    "cors(request, methods=['GET', 'PUT', 'OPTIONS'])",
                    "request.setHeader('Allow', 'GET, PUT, OPTIONS')",
                    'dbclient = SubscriberDB.getAllClients()',
                    'logging.debug(request.requestHeaders)',
                    'accept_type = request.requestHeaders.getRawHeaders("Accept")[0]',
                    'clients = ""',
                    'if not None:',
                    '    if accept_type == "application/json":',
                    '        request.setHeader("Content-Type", "application/json; charset=UTF-8")',
                    '        request.setResponseCode(200)',
                    '        for cl in dbclient:',
                    '            clients += str(\'{"id":"%s", "uri":"%s", "method":"%s", "accept":"%s"}, \' % (cl[0], cl[1], cl[2], cl[3]))',
                    "        return str('{\"clients\": {\"client\":[%s]}}' % (clients[:-2]))",
                    '    elif accept_type == "application/xml":',
                    '        request.setHeader("Content-Type", "application/xml; charset=UTF-8")',
                    '        request.setResponseCode(200)',
                    '        for cl in dbclient:',
                    "            clients += str('<client><id>%s</id><uri>%s</uri><method>%s</method><accept>%s</accept></client> ' % (cl[0], cl[1], cl[2], cl[3]))",
                    "        return str('<clients>%s</clients>' % (clients))",
                    '    else:',
                    '        request.write("<!DOCTYPE html>\\n")',
                    '        flattenString(request, ClientsElement(dbclient)).addCallback(request.write)',
                    '        request.finish()'
            ])
        ], ["xwot_app", node.name()])

        self._out.code([
            '#',
           "# %s '%s'" % ('POST', node.fullpath()),
           '#',
           "@app.route('%s', methods=['%s'])" % (node.fullpath(), 'POST'),
           "def handle%s_%s(request):" % (self._create_symbol_name(node.fullpath()), 'POST'),
            self._out.indent([
                    "cors(request, methods=['GET', 'PUT', 'OPTIONS'])",
                    "request.setHeader('Allow', 'GET, PUT, OPTIONS')",
                    'json_data = json.loads(request.content.getvalue())',
                    'logging.debug(json_data)',
                    'logging.debug(request.requestHeaders)',
                    'accept_type = request.requestHeaders.getRawHeaders("Accept")[0]',
                    "lastrowid = SubscriberDB.insertClient(json_data['uri'], json_data['method'], json_data['accept'], '1')",
                    'if not None:',
                    '    if accept_type == "application/json":',
                    '        request.setHeader("Content-Type", "application/json; charset=UTF-8")',
                    '        request.setResponseCode(200)',
                    "        return str('{\"id\":\"%s\", \"uri\":\"%s\", \"method\":\"%s\", \"accept\":\"%s\"}' % (lastrowid, json_data['uri'], json_data['method'], json_data['accept']))",
                    '    elif accept_type == "application/xml":',
                    '        request.setHeader("Content-Type", "application/xml; charset=UTF-8")',
                    '        request.setResponseCode(200)',
                    "        return str('<client><id>%s</id><uri>%s</uri><method>%s</method><accept>%s</accept></client>' % (lastrowid, json_data['uri'], json_data['method'], json_data['accept']))",
                    '    else:',
                    '        request.write("<!DOCTYPE html>\\n")',
                    "        flattenString(request, ClientElement(lastrowid, json_data['uri'], json_data['method'], json_data['accept'], '')).addCallback(request.write)",
                    '        request.finish()'
            ])
        ], ["xwot_app", node.name()])

        self._out.code([
            '#',
           "# %s '%s'" % ('WS', node.fullpath()),
           '#',
           "@app.route('%s/', branch=True)" % (node.fullpath()),
           "def handle%s_ws(request):" % (self._create_symbol_name(node.fullpath())),
            self._out.indent([
                'ServerFactory = xWoTBroadcastFactory',
                "shutter = SampleHardware()",
                'factory = ServerFactory("ws://localhost:5000/", shutter, debug = False,  debugCodePaths = False)',
                'factory.protocol = xWoTStreamerProtocol',
                'factory.setProtocolOptions(allowHixie76 = True)',
                'return WebSocketResource(factory)'
            ])
        ], ["xwot_app", node.name()])
        self._out.code([
            '#',
            "# %s '%s'" % ('OPTIONS', node.fullpath()),
            '#',
            "@app.route('%s', methods=['%s'])" % (node.fullpath(), 'OPTIONS'),
            "def handle%s_%s(request):" % (self._create_symbol_name(node.fullpath()), 'OPTIONS'),
            self._out.indent([
                "cors(request, methods=['GET', 'PUT', 'OPTIONS'])",
                "request.setHeader('Allow', 'GET, PUT, OPTIONS')"
            ])
         ], ["xwot_app", node.name()])

    def handle_publisher_client_resource(self, node):
        #nodename = 'ClientResource'.join(node._name.rsplit('Resource', 1))
        self._out.code([
            'from twisted.web.server import NOT_DONE_YET',
            'import logging',
            'from templates.tpl_client import ClientElement',
            'from twisted.web.template import Element, renderer, XMLFile, flattenString',
            'from  xwot.util.SubscriberDB import SubscriberDB',
        ], ["xwot_app", node.name()])

        self._out.code([
            '#'
            "# %s '%s'" % ('OPTIONS', node.fullpath()),
            '#',
            "@app.route('%s', methods=['%s'])" % (node.fullpath(), 'OPTIONS'),
            "def handle%s_%s(request, clientid):" % (self._create_symbol_name(node.fullpath()), 'OPTIONS'),
            self._out.indent([
                "cors(request, methods=['GET', 'PUT', 'OPTIONS'])",
                "request.setHeader('Allow', 'GET, PUT, OPTIONS')"
            ])
        ], ["xwot_app", node.name()])

        self._out.code([
            '#'
            "# %s '%s'" % ('GET', node.fullpath()),
            '#',
            "@app.route('%s', methods=['%s'])" % (node.fullpath(), 'GET'),
            "def handle%s_%s(request, clientid):" % (self._create_symbol_name(node.fullpath()), 'GET'),
            self._out.indent([
                "logging.debug(request.requestHeaders)",
                "cors(request, methods=['GET', 'PUT', 'OPTIONS'])",
                "accept_type = request.requestHeaders.getRawHeaders('Accept')[0].split(',')[0]",
                "client = SubscriberDB.getClient(clientid)",
                "if not None:",
                "    if 'application/json' in accept_type:",
                "        request.setHeader('Content-Type', 'application/json; charset=UTF-8')",
                "        request.setResponseCode(200)",
                "        return str('{\"id\":\"%s\", \"uri\":\"%s\", \"method\":\"%s\", \"accept\":\"%s\"}' % (client[0], client[1], client[2], client[3]))",
                "    elif accept_type == 'application/xml':",
                "        request.setHeader('Content-Type', 'application/xml; charset=UTF-8')",
                "        request.setResponseCode(200)",
                "        return str('<client><id>%s</id><uri>%s</uri><method>%s</method><accept>%s</accept></client>' % (client[0], client[1], client[2], client[3]))",
                "    else:",
                "        request.write('<!DOCTYPE html>\\n')",
                "        flattenString(request, ClientElement(client[0], client[1], client[2], client[3], '')).addCallback(request.write)",
                "        request.finish()",
            ])
        ], ["xwot_app", node.name()])

        self._out.code([
            '#'
            "# %s '%s'" % ('PUT', node.fullpath()),
            '#',
            "@app.route('%s', methods=['%s'])" % (node.fullpath(), 'PUT'),
            "def handle%s_%s(request, clientid):" % (self._create_symbol_name(node.fullpath()), 'PUT'),
            self._out.indent([
                'logging.debug(request.requestHeaders)',
                "cors(request, methods=['GET', 'PUT', 'OPTIONS'])",
                "request.setHeader('Allow', 'GET, PUT, OPTIONS')",
                'accept_type = request.requestHeaders.getRawHeaders("Accept")[0].split(',')[0]',
                '#TODO Update the client in the databse',
                'client = SubscriberDB.getClient(clientid)',
                'if not None:',
                '    if "application/json" in accept_type:',
                '        request.setHeader("Content-Type", "application/json; charset=UTF-8")',
                '        request.setResponseCode(200)',
                '        return str(\'{"id":"%s", "uri":"%s", "method":"%s", "accept":"%s"}\' % (client[0], client[1], client[2], client[3]))',
                '    elif "application/xml" in accept_type:',
                '        request.setHeader("Content-Type", "application/xml; charset=UTF-8")',
                '        request.setResponseCode(200)',
                '        return str(\'<client><id>%s</id><uri>%s</uri><method>%s</method><accept>%s</accept></client>\' % (client[0], client[1], client[2], client[3]))',
                '    else:',
                '        request.write("<!DOCTYPE html>\\n")',
                '        flattenString(request, ClientElement(client[0], client[1], client[2], client[3], '')).addCallback(request.write)',
                '        request.finish()',
            ])
        ], ["xwot_app", node.name()])

        self._out.code([
            '#'
            "# %s '%s'" % ('DELETE', node.fullpath()),
            '#',
            "@app.route('%s', methods=['%s'])" % (node.fullpath(), 'DELETE'),
            "def handle%s_%s(request, clientid):" % (self._create_symbol_name(node.fullpath()), 'DELETE'),
            self._out.indent([
                "cors(request, methods=['GET', 'PUT', 'OPTIONS'])",
                "request.setHeader('Allow', 'GET, PUT, OPTIONS')",
                'logging.debug(request.requestHeaders)',
                'accept_type = request.requestHeaders.getRawHeaders("Accept")[0].split(',')[0]',
                'client = SubscriberDB.getClient(clientid)',
                'SubscriberDB.deleteQuery(clientid)',
                'if not None:',
                '    if "application/json" in accept_type:',
                '        request.setHeader("Content-Type", "application/json; charset=UTF-8")',
                '        request.setResponseCode(200)',
                '        return str(\'{"id":"%s", "uri":"%s", "method":"%s", "accept":"%s"}\' % (client[0], client[1], client[2], client[3]))',
                '    elif "application/xml" in accept_type:',
                '        request.setHeader("Content-Type", "application/xml; charset=UTF-8")',
                '        request.setResponseCode(200)',
                '        return str(\'<client><id>%s</id><uri>%s</uri><method>%s</method><accept>%s</accept></client>\' % (client[0], client[1], client[2], client[3]))',
                '    else:',
                '        request.write("<!DOCTYPE html>\\n")',
                '        flattenString(request, ClientElement(client[0], client[1], client[2], client[3], '')).addCallback(request.write)',
                '        request.finish()',
            ])
        ], ["xwot_app", node.name()])

    def _runserver_tac(self):
        return """# coding: utf-8
#
# Generated by xwot compiler.
#
# Klein xwot application.
#

from twisted.internet import reactor
from twisted.web.server import Site
from twisted.application.service import Application
import xwot_app
from xwot_app import app

import yadp
from yadp import service
from yadp.device import Device

device = Device(urn='urn:xwot:Device', location=xwot_app.http_addr, descriptions=[xwot_app.yadp_description])

service = service()
service.register(device=device, passive=True)

application = Application('twisted-klein')
reactor.listenTCP(xwot_app.port, Site(app.resource()), interface='0.0.0.0')
        """

    def output(self):

        # get all created files
        files = self._out.flush()
        new_files_dic = {}

        for file_name, pycode in files.items():
            # append to each file its correct file extension
            new_files_dic[file_name + ".py"] = pycode

        # create on the fly a requirements.txt file
        requirements = "\n".join(['klein'])
        new_files_dic['requirements.txt'] = requirements
        new_files_dic['runserver.tac'] = self._runserver_tac()

        return new_files_dic


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# xwot.py - Python tools for the extended Web of Things
# Copyright (C) 2015  Alexander Rüedlinger
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

__author__ = 'Alexander Rüedlinger'

import argparse
import os.path
from xwot.compiler import Compiler
from xwot.compiler.frontend.processing import JSONLDDescriptionBuilder


def xwotc():
    """
    xwot compiler command
    """

    backends = Compiler.BACKENDS.keys()
    parser = argparse.ArgumentParser(description='xwot compiler')
    parser.add_argument('-p', dest='platform', type=str, default='klein', choices=backends, nargs='?',
                        help='platform to use')

    parser.add_argument('-o', dest='output_dir', type=str, default='out-app', nargs='?',
                        help='name of the output directory')

    parser.add_argument(dest='xwot_file', metavar='f', type=str,
                        help='xwot file')
    args = parser.parse_args()
    compiler = Compiler(input_file=args.xwot_file, output_dir=args.output_dir, platform=args.platform)
    compiler.compile()


def xwotd():
    """
    xwot description builder command
    """

    parser = argparse.ArgumentParser(description='xwot description builder')
    parser.add_argument('xwot_file', metavar='xwot file', type=str, nargs=1, help='a xwot file')

    parser.add_argument('-o', dest='output_filepath', type=str, default=None, nargs='?',
                        help='path of the output file')

    args = parser.parse_args()

    xml_filepath = args.xwot_file[0]
    description_builder = JSONLDDescriptionBuilder()
    out = description_builder.build(xml_file=xml_filepath)

    output_filepath = args.output_filepath
    if output_filepath is None:
        output_filepath = os.path.basename(xml_filepath) + ".jsonld"

    with open(output_filepath, "w+") as f:
        f.write(out)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# xwot.py - Python tools for the extended Web of Things
# Copyright (C) 2015  Alexander Rüedlinger
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

__author__ = 'Alexander Rüedlinger'

from setuptools import setup, find_packages

VERSION = "1.4.0"

setup(
    name="xwot-py",
    packages=find_packages('.', exclude=["tests", "test"]),
    entry_points={
        'console_scripts': [
            'xwotc = xwot.cmd:xwotc',
            'xwotd = xwot.cmd:xwotd'
        ]
    },
    version=VERSION,
    install_requires=['dicttoxml==1.6.6', 'xmltodict==0.9.2'],
    description="xwot",
    author="Alexander Rüedlinger",
    author_email="a.rueedlinger@gmail.com",
    license="GPLv3",
    classifiers=['License :: OSI Approved :: GNU General Public License v3 (GPLv3)'],
    ext_modules=[],
    long_description="""\
xwot.py
-------------------------------------

xwot.py - Python tools for the extended Web of Things

"""
)
